@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2020-2024 - pancake, thestr4ng3r */
+/* radare - LGPL - Copyright 2020-2025 - pancake, thestr4ng3r */
 
 #include "r2r.h"
 
@@ -418,6 +418,17 @@ static void handle_sigchld(int sig) {
 	}
 }
 
+static R2RSubprocess *pid_to_proc(int pid) {
+	void **it;
+	r_pvector_foreach (&subprocs, it) {
+		R2RSubprocess *p = *it;
+		if (p->pid == pid) {
+			return p;
+		}
+	}
+	return NULL;
+}
+
 static RThreadFunctionRet sigchld_th(RThread *th) {
 	while (true) {
 		ut8 b;
@@ -436,37 +447,42 @@ static RThreadFunctionRet sigchld_th(RThread *th) {
 		}
 		while (true) {
 			int wstat;
-			pid_t pid = waitpid (-1, &wstat, WNOHANG);
+			// pid_t pid = wait (&wstat);
+			pid_t pid = waitpid (-1, &wstat, 0);
 			if (pid <= 0) {
+			// 	r_sys_perror ("waitpid failed");
 				break;
 			}
 			r_th_lock_enter (subprocs_mutex);
-			void **it;
-			R2RSubprocess *proc = NULL;
-			r_pvector_foreach (&subprocs, it) {
-				R2RSubprocess *p = *it;
-				if (p->pid == pid) {
-					proc = p;
-					break;
-				}
-			}
+			R2RSubprocess *proc = pid_to_proc (pid);
 			if (!proc) {
-			 	r_th_lock_leave (subprocs_mutex);
+				r_th_lock_leave (subprocs_mutex);
 				continue;
 			}
-			r_th_lock_enter (proc->lock);
-			if (WIFEXITED (wstat)) {
-				proc->ret = WEXITSTATUS (wstat);
-			} else {
-				proc->ret = -1;
+			// Capture exit status while holding only one lock
+			int exit_status = -1;
+#if !__wasi__
+			if (WIFSIGNALED (wstat)) {
+				const int signal_number = WTERMSIG (wstat);
+				R_LOG_ERROR ("Child signal %d", signal_number);
+				exit_status = -1;
+			} else if (WIFEXITED (wstat)) {
+				exit_status = WEXITSTATUS (wstat);
 			}
-			ut8 r = 0;
-			int ret = write (proc->killpipe[1], &r, 1);
+#endif
+
+			// Update process status before signaling
+			r_th_lock_enter (proc->lock);
+			proc->ret = exit_status;
 			r_th_lock_leave (proc->lock);
-			r_th_lock_leave (subprocs_mutex);
+			// Signal process completion through killpipe
+			int ret = write (proc->killpipe[1], "", 1);
 			if (ret != 1) {
+				r_sys_perror ("write killpipe-");
+				r_th_lock_leave (subprocs_mutex);
 				break;
 			}
+			r_th_lock_leave (subprocs_mutex);
 		}
 	}
 	return R_TH_STOP;
@@ -485,22 +501,22 @@ R_API bool r2r_subprocess_init(void) {
 	}
 	sigchld_thread = r_th_new (sigchld_th, NULL, 0);
 	if (!r_th_start (sigchld_thread)) {
-		if (sigchld_thread) {
-			r_th_free (sigchld_thread);
-			sigchld_thread = NULL;
-		}
-		close (sigchld_pipe [0]);
-		close (sigchld_pipe [1]);
-		r_th_lock_free (subprocs_mutex);
-		return false;
+		goto error;
 	}
 	if (r_sys_signal (SIGCHLD, handle_sigchld) < 0) {
-		close (sigchld_pipe [0]);
-		close (sigchld_pipe [1]);
-		r_th_lock_free (subprocs_mutex);
-		return false;
+		goto error;
 	}
 	return true;
+
+error:
+	if (sigchld_thread) {
+		r_th_free (sigchld_thread);
+		sigchld_thread = NULL;
+	}
+	close (sigchld_pipe [0]);
+	close (sigchld_pipe [1]);
+	r_th_lock_free (subprocs_mutex);
+	return false;
 }
 
 R_API void r2r_subprocess_fini(void) {
@@ -517,18 +533,21 @@ R_API void r2r_subprocess_fini(void) {
 	r_th_lock_free (subprocs_mutex);
 }
 
+static inline void dup_retry(int fds[2], int n, int b) {
+	while ((dup2 (fds[n], b) == -1) && (errno == EINTR)) {
+		;
+	}
+	close (fds[0]);
+	close (fds[1]);
+}
+
 R_API R2RSubprocess *r2r_subprocess_start(
 		const char *file, const char *args[], size_t args_size,
 		const char *envvars[], const char *envvals[], size_t env_size) {
-	char **argv = calloc (args_size + 2, sizeof (char *));
-	if (!argv) {
-		return NULL;
-	}
-	argv[0] = (char *)file;
-	if (args_size) {
-		memcpy (argv + 1, args, sizeof (char *) * args_size);
-	}
-	// done by calloc: argv[args_size + 1] = NULL;
+	int stdin_pipe[2] = { -1, -1 };
+	int stdout_pipe[2] = { -1, -1 };
+	int stderr_pipe[2] = { -1, -1 };
+
 	r_th_lock_enter (subprocs_mutex);
 	R2RSubprocess *proc = R_NEW0 (R2RSubprocess);
 	if (!proc) {
@@ -549,30 +568,27 @@ R_API R2RSubprocess *r2r_subprocess_start(
 		goto error;
 	}
 
-	int stdin_pipe[2] = { -1, -1 };
 	if (pipe (stdin_pipe) == -1) {
 		r_sys_perror ("subproc-start pipe");
 		goto error;
 	}
 	proc->stdin_fd = stdin_pipe[1];
 
-	int stdout_pipe[2] = { -1, -1 };
 	if (pipe (stdout_pipe) == -1) {
 		r_sys_perror ("subproc-start pipe");
 		goto error;
 	}
-	if (fcntl(stdout_pipe[0], F_SETFL, O_NONBLOCK) < 0) {
+	if (fcntl (stdout_pipe[0], F_SETFL, O_NONBLOCK) < 0) {
 		r_sys_perror ("subproc-start fcntl");
 		goto error;
 	}
 	proc->stdout_fd = stdout_pipe[0];
 
-	int stderr_pipe[2] = { -1, -1 };
 	if (pipe (stderr_pipe) == -1) {
 		r_sys_perror ("subproc-start pipe");
 		goto error;
 	}
-	if (fcntl(stderr_pipe[0], F_SETFL, O_NONBLOCK) < 0) {
+	if (fcntl (stderr_pipe[0], F_SETFL, O_NONBLOCK) < 0) {
 		r_sys_perror ("subproc-start fcntl");
 		goto error;
 	}
@@ -584,21 +600,21 @@ R_API R2RSubprocess *r2r_subprocess_start(
 		r_th_lock_leave (subprocs_mutex);
 		r_sys_perror ("subproc-start fork");
 		free (proc);
-		free (argv);
 		return NULL;
 	}
 	if (proc->pid == 0) {
-		// child
-		while ((dup2 (stdin_pipe[0], STDIN_FILENO) == -1) && (errno == EINTR)) {}
-		close (stdin_pipe[0]);
-		close (stdin_pipe[1]);
-		while ((dup2 (stdout_pipe[1], STDOUT_FILENO) == -1) && (errno == EINTR)) {}
-		close (stdout_pipe[1]);
-		close (stdout_pipe[0]);
-		while ((dup2 (stderr_pipe[1], STDERR_FILENO) == -1) && (errno == EINTR)) {}
-		close (stderr_pipe[1]);
-		close (stderr_pipe[0]);
-
+		dup_retry (stdin_pipe, 0, STDIN_FILENO);
+		dup_retry (stdout_pipe, 1, STDOUT_FILENO);
+		dup_retry (stderr_pipe, 1, STDERR_FILENO);
+		char **argv = calloc (args_size + 2, sizeof (char *));
+		if (!argv) {
+			free (proc);
+			return NULL;
+		}
+		argv[0] = (char *)file;
+		if (args_size) {
+			memcpy (argv + 1, args, sizeof (char *) * args_size);
+		}
 		size_t i;
 		for (i = 0; i < env_size; i++) {
 			setenv (envvars[i], envvals[i], 1);
@@ -607,7 +623,6 @@ R_API R2RSubprocess *r2r_subprocess_start(
 		r_sys_perror ("subproc-start exec");
 		r_sys_exit (-1, true);
 	}
-	free (argv);
 
 	// parent
 	close (stdin_pipe[0]);
@@ -620,7 +635,6 @@ R_API R2RSubprocess *r2r_subprocess_start(
 
 	return proc;
 error:
-	free (argv);
 	if (proc && proc->killpipe[0] == -1) {
 		close (proc->killpipe[0]);
 	}
@@ -710,6 +724,7 @@ R_API bool r2r_subprocess_wait(R2RSubprocess *proc, ut64 timeout_ms) {
 			ssize_t sz = read (proc->stdout_fd, buf, sizeof (buf));
 			if (sz < 0) {
 				r_sys_perror ("sp-wait read 1");
+				child_dead = true;
 				stdout_eof = true;
 			} else if (sz == 0) {
 				stdout_eof = true;
@@ -723,8 +738,10 @@ R_API bool r2r_subprocess_wait(R2RSubprocess *proc, ut64 timeout_ms) {
 			ssize_t sz = read (proc->stderr_fd, buf, sizeof (buf));
 			if (sz < 0) {
 				r_sys_perror ("sp-wait read 2");
+				timedout = false;
 				stderr_eof = true;
-				continue;
+				child_dead = true;
+				break;
 			}
 			if (sz == 0) {
 				stderr_eof = true;
@@ -761,15 +778,27 @@ R_API void r2r_subprocess_stdin_write(R2RSubprocess *proc, const ut8 *buf, size_
 }
 
 R_API R2RProcessOutput *r2r_subprocess_drain(R2RSubprocess *proc) {
-	r_th_lock_enter (subprocs_mutex);
-	R2RProcessOutput *out = R_NEW (R2RProcessOutput);
-	if (out) {
+	if (!proc) {
+		return NULL;
+	}
+	R2RProcessOutput *out = R_NEW0 (R2RProcessOutput);
+	if (proc->lock && r_th_lock_enter (proc->lock)) {
 		out->out = r_strbuf_drain_nofree (&proc->out);
 		out->err = r_strbuf_drain_nofree (&proc->err);
 		out->ret = proc->ret;
 		out->timeout = false;
+		r_th_lock_leave (proc->lock);
+	} else {
+		// If we can't acquire the lock, make safe copies
+		if (proc->out.ptr) {
+			out->out = strdup (r_strbuf_get (&proc->out));
+		}
+		if (proc->err.ptr) {
+			out->err = strdup (r_strbuf_get (&proc->err));
+		}
+		out->ret = proc->ret;
+		out->timeout = false;
 	}
-	r_th_lock_leave (subprocs_mutex);
 	return out;
 }
 
@@ -777,19 +806,57 @@ R_API void r2r_subprocess_free(R2RSubprocess *proc) {
 	if (!proc) {
 		return;
 	}
-	r_th_lock_enter (subprocs_mutex);
-	r_th_lock_free (proc->lock);
+	// Take mutex to safely modify the subprocs vector
+	if (!r_th_lock_enter (subprocs_mutex)) {
+		// If we can't take the lock, still try to free resources
+		// to avoid leaking, but don't modify shared data structures
+		goto cleanup_without_vector;
+	}
+	// Remove from global vector of subprocesses
 	r_pvector_remove_data (&subprocs, proc);
 	r_th_lock_leave (subprocs_mutex);
-	r_strbuf_fini (&proc->out);
-	r_strbuf_fini (&proc->err);
-	close (proc->killpipe[0]);
-	close (proc->killpipe[1]);
+	// Now safely clean up process resources
+cleanup_without_vector:
+	// Acquire the process lock to ensure no one is currently
+	// writing to or reading from its buffers
+	if (proc->lock) {
+		r_th_lock_enter (proc->lock);
+		// Free buffers - only reinitialize them if they haven't been drained
+		// This prevents double frees when r2r_subprocess_drain has been called
+		if (proc->out.ptr) {
+			r_strbuf_fini (&proc->out);
+			r_strbuf_init (&proc->out); // Reinitialize to avoid issues with subsequent r_strbuf_fini
+		}
+		if (proc->err.ptr) {
+			r_strbuf_fini (&proc->err);
+			r_strbuf_init (&proc->err); // Reinitialize to avoid issues with subsequent r_strbuf_fini
+		}
+		// Release the process lock before freeing it
+		r_th_lock_leave (proc->lock);
+		r_th_lock_free (proc->lock);
+	} else {
+		// Even if we can't get the lock, we need to safely clean up buffers
+		// If buffers have been drained, ptr would be NULL and this is safe
+		r_strbuf_fini (&proc->out);
+		r_strbuf_fini (&proc->err);
+	}
+	// Close all open file descriptors
+	if (proc->killpipe[0] != -1) {
+		close (proc->killpipe[0]);
+	}
+	if (proc->killpipe[1] != -1) {
+		close (proc->killpipe[1]);
+	}
 	if (proc->stdin_fd != -1) {
 		close (proc->stdin_fd);
 	}
-	close (proc->stdout_fd);
-	close (proc->stderr_fd);
+	if (proc->stdout_fd != -1) {
+		close (proc->stdout_fd);
+	}
+	if (proc->stderr_fd != -1) {
+		close (proc->stderr_fd);
+	}
+	// Finally free the process struct itself
 	free (proc);
 }
 #endif
@@ -929,7 +996,6 @@ static R2RProcessOutput *run_r2_test(R2RRunConfig *config, ut64 timeout_ms, int
 		r_pvector_push (&envvars, "R2_NOPLUGINS");
 		r_pvector_push (&envvals, "1");
 	}
-	
 	if (extra_env) {
 		RListIter *eit;
 		char *kv;
@@ -1177,7 +1243,7 @@ R_API R2RAsmTestOutput *r2r_run_asm_test(R2RRunConfig *config, R2RAsmTest *test)
 		if (!bytes) {
 			goto rip;
 		}
-		int byteslen = r_hex_str2bin (hex, bytes);
+		const int byteslen = r_hex_str2bin (hex, bytes);
 		if (byteslen <= 0) {
 			free (bytes);
 			goto rip;
@@ -1189,6 +1255,9 @@ rip:
 		r2r_subprocess_free (proc);
 	}
 	if (test->mode & R2R_ASM_TEST_MODE_DISASSEMBLE) {
+		if (test->bytes_size < 1) {
+			goto beach;
+		}
 		char *hex = r_hex_bin2strdup (test->bytes, test->bytes_size);
 		if (!hex) {
 			goto beach;
@@ -1387,7 +1456,7 @@ R_API R2RTestResultInfo *r2r_run_test(R2RRunConfig *config, R2RTest *test) {
 	int needsabi = r2r_test_needsabi (test);
 	switch (test->type) {
 	case R2R_TEST_TYPE_CMD:
-		if (r_sys_getenv_asbool ("R2R_SKIP_CMD")) {
+		if (config->skip_cmd) {
 			success = true;
 			ret->run_failed = false;
 		} else {
@@ -1419,7 +1488,7 @@ R_API R2RTestResultInfo *r2r_run_test(R2RRunConfig *config, R2RTest *test) {
 		}
 		break;
 	case R2R_TEST_TYPE_ASM:
-		if (r_sys_getenv_asbool ("R2R_SKIP_ASM")) {
+		if (config->skip_asm) {
 			success = true;
 			ret->run_failed = false;
 		} else {
@@ -1428,12 +1497,17 @@ R_API R2RTestResultInfo *r2r_run_test(R2RRunConfig *config, R2RTest *test) {
 			success = r2r_check_asm_test (out, at);
 			const bool is_broken = at->mode & R2R_ASM_TEST_MODE_BROKEN;
 			if (!success && !is_broken) {
-				char *b0 = r_hex_bin2strdup (at->bytes, at->bytes_size);
-				char *b1 = r_hex_bin2strdup (out->bytes, out->bytes_size);
-				eprintf ("\n"Color_RED"- %s"Color_RESET" # %s\n", at->disasm, b0);
-				eprintf (Color_GREEN"+ %s"Color_RESET" # %s\n", out->disasm, b1);
-				free (b0);
-				free (b1);
+				if (at->bytes_size < 1 || out->bytes_size < 1) {
+					eprintf ("\n"Color_RED"- %s"Color_RESET" # bytes_size = %d\n", at->disasm, (int)at->bytes_size);
+					eprintf (Color_GREEN"+ %s"Color_RESET" # bytes_size = %d\n", out->disasm, (int)out->bytes_size);
+				} else {
+					char *b0 = r_hex_bin2strdup (at->bytes, at->bytes_size);
+					char *b1 = r_hex_bin2strdup (out->bytes, out->bytes_size);
+					eprintf ("\n"Color_RED"- %s"Color_RESET" # %s\n", at->disasm, b0);
+					eprintf (Color_GREEN"+ %s"Color_RESET" # %s\n", out->disasm, b1);
+					free (b0);
+					free (b1);
+				}
 			}
 			// TODO: show more details of the failed assembled instruction
 			ret->asm_out = out;
@@ -1442,7 +1516,7 @@ R_API R2RTestResultInfo *r2r_run_test(R2RRunConfig *config, R2RTest *test) {
 		}
 		break;
 	case R2R_TEST_TYPE_JSON:
-		if (r_sys_getenv_asbool ("R2R_SKIP_JSON")) {
+		if (config->skip_json) {
 			success = true;
 			ret->run_failed = false;
 		} else {
@@ -1465,7 +1539,7 @@ R_API R2RTestResultInfo *r2r_run_test(R2RRunConfig *config, R2RTest *test) {
 		}
 		break;
 	case R2R_TEST_TYPE_FUZZ:
-		if (r_sys_getenv_asbool ("R2R_SKIP_FUZZ")) {
+		if (config->skip_fuzz) {
 			success = true;
 			ret->run_failed = false;
 		} else {
