@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2019-2024 - pancake, thestr4ng3r */
+/* radare - LGPL - Copyright 2019-2025 - pancake, thestr4ng3r */
 
 #include <r_anal.h>
 
@@ -73,13 +73,12 @@ static void label_addrs_kv_free(HtPPKv *kv) {
 }
 
 R_API RAnalFunction *r_anal_function_new(RAnal *anal) {
+	// XXX fcn->name is null because its r_anal_create_function the one that must be called
+	R_RETURN_VAL_IF_FAIL (anal, NULL);
 	RAnalFunction *fcn = R_NEW0 (RAnalFunction);
-	if (!fcn) {
-		return NULL;
-	}
 	fcn->anal = anal;
 	fcn->addr = UT64_MAX;
-	fcn->cc = r_str_constpool_get (&anal->constpool, r_anal_cc_default (anal));
+	fcn->callconv = r_str_constpool_get (&anal->constpool, r_anal_cc_default (anal));
 	fcn->bits = anal->config->bits;
 	fcn->bbs = r_list_new ();
 	fcn->diff = r_anal_diff_new ();
@@ -91,6 +90,7 @@ R_API RAnalFunction *r_anal_function_new(RAnal *anal) {
 	fcn->inst_vars = ht_up_new (NULL, inst_vars_kv_free, NULL);
 	fcn->labels = ht_up_new (NULL, labels_kv_free, NULL);
 	fcn->label_addrs = ht_pp_new (NULL, label_addrs_kv_free, NULL);
+	fcn->ts = r_time_now ();
 	return fcn;
 }
 
@@ -138,7 +138,12 @@ R_API bool r_anal_add_function(RAnal *anal, RAnalFunction *fcn) {
 		R_LOG_WARN ("Unnamed function at 0x%08"PFMT64x, fcn->addr);
 		// r_sys_breakpoint ();
 		free (fcn->name);
-		fcn->name = r_str_newf ("fcn.%"PFMT64x, fcn->addr);
+		const char *fcnprefix = anal->coreb.cfgGet ? anal->coreb.cfgGet (anal->coreb.core, "anal.fcnprefix") : NULL;
+		if (R_STR_ISEMPTY (fcnprefix)) {
+			fcn->name = r_str_newf ("fcn_%08"PFMT64x, fcn->addr);
+		} else {
+			fcn->name = r_str_newf ("%s.%"PFMT64x, fcnprefix, fcn->addr);
+		}
 	}
 	if (__fcn_exists (anal, fcn->name, fcn->addr)) {
 		return false;
@@ -153,28 +158,30 @@ R_API bool r_anal_add_function(RAnal *anal, RAnalFunction *fcn) {
 	r_list_append (anal->fcns, fcn);
 	ht_pp_insert (anal->ht_name_fun, fcn->name, fcn);
 	ht_up_insert (anal->ht_addr_fun, fcn->addr, fcn);
+	{
+		REventFunction event = { .addr = fcn->addr, .fcn = fcn };
+		r_event_send (anal->ev, R_EVENT_FUNCTION_ADDED, &event);
+	}
 	return true;
 }
 
 R_API RAnalFunction *r_anal_create_function(RAnal *anal, const char *name, ut64 addr, int type, RAnalDiff *diff) {
 	R_RETURN_VAL_IF_FAIL (anal && addr != UT64_MAX, NULL);
 	RAnalFunction *fcn = r_anal_function_new (anal);
-	if (!fcn) {
-		return NULL;
-	}
 	fcn->addr = addr;
 	fcn->type = type;
-	fcn->cc = r_str_constpool_get (&anal->constpool, r_anal_cc_default (anal));
+	fcn->callconv = r_str_constpool_get (&anal->constpool, r_anal_cc_default (anal));
 	fcn->bits = anal->config->bits;
 	if (name) {
 		free (fcn->name);
 		fcn->name = strdup (name);
 	} else {
 		const char *fcnprefix = anal->coreb.cfgGet ? anal->coreb.cfgGet (anal->coreb.core, "anal.fcnprefix") : NULL;
-		if (!fcnprefix) {
-			fcnprefix = "fcn";
+		if (R_STR_ISEMPTY (fcnprefix)) {
+			fcn->name = r_str_newf ("fcn_%08"PFMT64x, fcn->addr);
+		} else {
+			fcn->name = r_str_newf ("%s.%08"PFMT64x, fcnprefix, fcn->addr);
 		}
-		fcn->name = r_str_newf ("%s.%08"PFMT64x, fcnprefix, fcn->addr);
 	}
 	if (diff) {
 		fcn->diff->type = diff->type;
@@ -191,9 +198,15 @@ R_API RAnalFunction *r_anal_create_function(RAnal *anal, const char *name, ut64
 	return fcn;
 }
 
-R_API bool r_anal_function_delete(RAnalFunction *fcn) {
+R_API bool r_anal_function_delete(RAnal *anal, RAnalFunction *fcn) {
 	R_RETURN_VAL_IF_FAIL (fcn, false);
-	return r_list_delete_data (fcn->anal->fcns, fcn);
+	ut64 fcn_addr = fcn->addr;
+	bool found = r_list_delete_data (fcn->anal->fcns, fcn);
+	if (found) {
+		REventFunction event = { .addr = fcn_addr };
+		r_event_send (anal->ev, R_EVENT_FUNCTION_DELETED, &event);
+	}
+	return found;
 }
 
 R_API RAnalFunction *r_anal_get_function_at(RAnal *anal, ut64 addr) {
@@ -240,6 +253,11 @@ R_API bool r_anal_function_rename(RAnalFunction *fcn, const char *name) {
 			// only re-insert if it really was in the tree before
 			ht_pp_insert (anal->ht_name_fun, fcn->name, fcn);
 		}
+		{
+			// TODO: maybe we want to know which was the old name?
+			REventFunction event = { .addr = fcn->addr, .fcn = fcn };
+			r_event_send (anal->ev, R_EVENT_FUNCTION_RENAMED, &event);
+		}
 		return true;
 	}
 	return false;
@@ -304,21 +322,25 @@ static void ensure_fcn_range(RAnalFunction *fcn) {
 }
 
 R_API ut64 r_anal_function_linear_size(RAnalFunction *fcn) {
+	R_RETURN_VAL_IF_FAIL (fcn, 0);
 	ensure_fcn_range (fcn);
 	return fcn->meta._max - fcn->meta._min;
 }
 
 R_API ut64 r_anal_function_min_addr(RAnalFunction *fcn) {
+	R_RETURN_VAL_IF_FAIL (fcn, 0);
 	ensure_fcn_range (fcn);
 	return fcn->meta._min;
 }
 
 R_API ut64 r_anal_function_max_addr(RAnalFunction *fcn) {
+	R_RETURN_VAL_IF_FAIL (fcn, 0);
 	ensure_fcn_range (fcn);
 	return fcn->meta._max;
 }
 
 R_API ut64 r_anal_function_size_from_entry(RAnalFunction *fcn) {
+	R_RETURN_VAL_IF_FAIL (fcn, 0);
 	ensure_fcn_range (fcn);
 	return fcn->meta._min == UT64_MAX ? 0 : fcn->meta._max - fcn->addr;
 }
@@ -367,7 +389,11 @@ R_API bool r_anal_function_was_modified(RAnalFunction *fcn) {
 }
 
 R_API int r_anal_function_coverage(RAnalFunction *fcn) {
+	R_RETURN_VAL_IF_FAIL (fcn, 0);
 	int total = r_list_length (fcn->bbs);
+	if (total == 0) {
+		return 0;
+	}
 	RListIter *iter;
 	RAnalBlock *bb;
 	int traced = 0;
