@@ -17,7 +17,7 @@
 
 #if CAPSTONE_HAS_M68K
 #include <capstone/m68k.h>
-// http://www.mrc.uidaho.edu/mrc/people/jff/digital/M68Kir.html
+// https://www.mrc.uidaho.edu/mrc/people/jff/digital/M68Kir.html
 
 // clang-format off
 // Source: https://wiki.neogeodev.org/index.php?title=68k_instructions_timings
@@ -251,7 +251,7 @@ static inline csh cs_handle_for_session(RArchSession *as) {
 	return pd->cs_handle;
 }
 
-static bool decode(RArchSession *as, RAnalOp *op, RAnalOpMask mask) {
+static bool decode(RArchSession *as, RAnalOp *op, RArchDecodeMask mask) {
 	const ut64 addr = op->addr;
 	const ut8 *buf = op->bytes;
 	const int len = op->size;
@@ -926,8 +926,8 @@ static bool encode(RArchSession *s, RAnalOp *op, ut32 mask) {
 const RArchPlugin r_arch_plugin_m68k_cs = {
 	.meta = {
 		.name = "m68k",
-		.desc = "Capstone M68K analyzer",
-		.license = "BSD",
+		.desc = "Motorola 680x0 (capstone)",
+		.license = "BSD-3-Clause",
 	},
 	.cpus = "68000,68010,68020,68030,68040,68060",
 	.arch = "m68k",
@@ -942,11 +942,11 @@ const RArchPlugin r_arch_plugin_m68k_cs = {
 };
 #else
 const RArchPlugin r_arch_plugin_m68k_cs = {
-	.name = "m68k (unsupported)",
-	.desc = "Capstone M68K analyzer (unsupported)",
-	.license = "BSD",
-	.arch = "m68k",
-	.bits = R_SYS_BITS_PACK1 (32),
+	.meta = {
+		.name = "m68k (unsupported)",
+		.desc = "Motorola 680x0 (capstone)",
+		.license = "BSD-3-Clause",
+	},
 };
 #endif
 
