@@ -47,15 +47,13 @@ static const char *__getname(RBin *bin, int type, int idx, bool sd) {
 // TODO: move these two function do a different file
 R_API RBinXtrData *r_bin_xtrdata_new(RBuffer *buf, ut64 offset, ut64 size, ut32 file_count, RBinXtrMetadata *metadata) {
 	RBinXtrData *data = R_NEW0 (RBinXtrData);
-	if (data) {
-		data->offset = offset;
-		data->size = size;
-		data->file_count = file_count;
-		data->metadata = metadata;
-		data->loaded = false;
-		// don't slice twice TODO. review this
-		data->buf = r_buf_ref (buf); // r_buf_new_slice (buf, offset, size);
-	}
+	data->offset = offset;
+	data->size = size;
+	data->file_count = file_count;
+	data->metadata = metadata;
+	data->loaded = false;
+	// don't slice twice TODO. review this
+	data->buf = r_buf_ref (buf); // r_buf_new_slice (buf, offset, size);
 	return data;
 }
 
@@ -104,7 +102,7 @@ R_API void r_bin_file_options_init(RBinFileOptions *opt, int fd, ut64 baseaddr,
 	opt->rawstr = rawstr;
 }
 
-R_API void r_bin_arch_options_init(RBinArchOptions *opt, R_NULLABLE const char *arch, int bits) {
+R_API void r_bin_arch_options_init(RBinArchOptions *opt, const char * R_NULLABLE arch, int bits) {
 	R_RETURN_IF_FAIL (opt);
 	opt->arch = arch? arch: R_SYS_ARCH;
 	opt->bits = bits? bits: R_SYS_BITS;
@@ -152,6 +150,7 @@ R_API RBinImport *r_bin_import_clone(RBinImport *o) {
 	RBinImport *res = r_mem_dup (o, sizeof (*o));
 	if (res) {
 		res->name = r_bin_name_clone (o->name);
+		res->libname = R_STR_DUP (o->libname);
 		res->classname = R_STR_DUP (o->classname);
 		res->descriptor = R_STR_DUP (o->descriptor);
 	}
@@ -171,11 +170,9 @@ R_API void r_bin_import_free(RBinImport *imp) {
 R_API RBinSymbol *r_bin_symbol_new(const char *name, ut64 paddr, ut64 vaddr) {
 	R_RETURN_VAL_IF_FAIL (name, NULL);
 	RBinSymbol *sym = R_NEW0 (RBinSymbol);
-	if (sym) {
-		sym->name = r_bin_name_new (name);
-		sym->paddr = paddr;
-		sym->vaddr = vaddr;
-	}
+	sym->name = r_bin_name_new (name);
+	sym->paddr = paddr;
+	sym->vaddr = vaddr;
 	return sym;
 }
 
@@ -195,8 +192,8 @@ R_API RBinSymbol *r_bin_symbol_clone(RBinSymbol *bs) {
 }
 
 // query the symbol name into the symtypes database
-R_API const char *r_bin_symbol_unsafe(RBin *bin, const char *name) {
-	Sdb *db = sdb_ns (bin->sdb, "symclass", true);
+R_API const char *r_bin_import_tags(RBin *bin, const char *name) {
+	Sdb *db = sdb_ns (bin->sdb, "symclass", true); // R2_600 - rename to imptags
 	if (db) {
 		return sdb_const_get (db, name, 0);
 	}
@@ -261,7 +258,7 @@ R_API bool r_bin_reload(RBin *bin, ut32 bf_id, ut64 baseaddr) {
 		return false;
 	}
 	RBinFileOptions opt;
-	r_bin_file_options_init (&opt, bf->fd, baseaddr, bf->loadaddr, bin->rawstr);
+	r_bin_file_options_init (&opt, bf->fd, baseaddr, bf->loadaddr, bin->options.rawstr);
 	opt.filename = bf->file;
 	if (!bf->buf) {
 		r_bin_file_delete (bin, bf->id);
@@ -278,14 +275,14 @@ R_API bool r_bin_open_buf(RBin *bin, RBuffer *buf, RBinFileOptions *opt) {
 	RListIter *it;
 	RBinXtrPlugin *xtr;
 
-	bin->rawstr = opt->rawstr;
+	bin->options.rawstr = opt->rawstr;
 	bin->file = opt->filename;
 	if (opt->loadaddr == UT64_MAX) {
 		opt->loadaddr = 0;
 	}
 
 	RBinFile *bf = NULL;
-	if (bin->use_xtr && !opt->pluginname) {
+	if (bin->options.use_xtr && !opt->pluginname) {
 		// XXX - for the time being this is fine, but we may want to
 		// change the name to something like
 		// <xtr_name>:<bin_type_name>
@@ -299,16 +296,15 @@ R_API bool r_bin_open_buf(RBin *bin, RBuffer *buf, RBinFileOptions *opt) {
 				    xtr->extract_from_bytes || xtr->extractall_from_bytes) {
 					bf = r_bin_file_xtr_load (bin, xtr,
 						bin->file, buf, opt->baseaddr, opt->loadaddr,
-						opt->xtr_idx, opt->fd, bin->rawstr);
+						opt->xtr_idx, opt->fd, bin->options.rawstr);
 				}
 			}
 		}
 	}
 	if (!bf) {
-		// Uncomment for this speedup: 20s vs 22s
-		// RBuffer *buf = r_buf_new_slurp (bin->file);
-		bf = r_bin_file_new_from_buffer (bin, bin->file? bin->file: "?", buf, bin->rawstr,
-			opt->baseaddr, opt->loadaddr, opt->fd, opt->pluginname);
+		const char *bfile = bin->file? bin->file: "?";
+		opt->rawstr = bin->options.rawstr;
+		bf = r_bin_file_new_from_buffer (bin, bfile, buf, opt);
 		if (!bf) {
 			return false;
 		}
@@ -569,6 +565,7 @@ static void __printXtrPluginDetails(RBin *bin, RBinXtrPlugin *bx, int json) {
 	}
 }
 
+// TODO: move to libr/core/clist
 R_API bool r_bin_list_plugin(RBin *bin, const char* name, PJ *pj, int json) {
 	RListIter *it;
 	RBinPlugin *bp;
@@ -577,13 +574,13 @@ R_API bool r_bin_list_plugin(RBin *bin, const char* name, PJ *pj, int json) {
 	R_RETURN_VAL_IF_FAIL (bin && name, false);
 
 	r_list_foreach (bin->plugins, it, bp) {
-		if (r_str_startswith (bp->meta.name, name)) {
+		if (!r_str_startswith (bp->meta.name, name)) {
 			continue;
 		}
 		return r_bin_print_plugin_details (bin, bp, pj, json);
 	}
 	r_list_foreach (bin->binxtrs, it, bx) {
-		if (r_str_startswith (bx->meta.name, name)) {
+		if (!r_str_startswith (bx->meta.name, name)) {
 			continue;
 		}
 		__printXtrPluginDetails (bin, bx, json);
@@ -612,7 +609,7 @@ R_API void r_bin_list(RBin *bin, PJ *pj, int format) {
 		r_list_foreach (bin->binxtrs, it, bx) {
 			bin->cb_printf ("%s\n", bx->meta.name);
 		}
-	} else if (format) {
+	} else if (pj) {
 		pj_o (pj);
 		pj_ka (pj, "bin");
 		r_list_foreach (bin->plugins, it, bp) {
@@ -644,20 +641,16 @@ R_API void r_bin_list(RBin *bin, PJ *pj, int format) {
 		pj_end (pj);
 	} else {
 		r_list_foreach (bin->plugins, it, bp) {
-			bin->cb_printf ("bin  %-11s %s (%s) %s %s\n",
-				bp->meta.name, bp->meta.desc, r_str_get_fail (bp->meta.license, "???"),
-				r_str_get (bp->meta.version),
-				r_str_get (bp->meta.author));
+			bin->cb_printf ("bin  %-11s %s\n", bp->meta.name, bp->meta.desc);
+			// bin->cb_printf ("bin  %-11s %s %s %s\n", bp->meta.name, bp->meta.desc, r_str_get (bp->meta.version), r_str_get (bp->meta.author));
 		}
 		r_list_foreach (bin->binxtrs, it, bx) {
 			const char *name = strncmp (bx->meta.name, "xtr.", 4)? bx->meta.name : bx->meta.name + 3;
-			bin->cb_printf ("xtr  %-11s %s (%s)\n", name,
-				bx->meta.desc, r_str_get_fail (bx->meta.license, "???"));
+			bin->cb_printf ("xtr  %-11s %s\n", name, bx->meta.desc);
 		}
 		r_list_foreach (bin->binldrs, it, ld) {
 			const char *name = strncmp (ld->meta.name, "ldr.", 4)? ld->meta.name : ld->meta.name + 3;
-			bin->cb_printf ("ldr  %-11s %s (%s)\n", name,
-				ld->meta.desc, r_str_get_fail (ld->meta.license, "???"));
+			bin->cb_printf ("ldr  %-11s %s\n", name, ld->meta.desc);
 		}
 	}
 	if (local_pj) {
@@ -762,7 +755,7 @@ R_API RBinSection *r_bin_get_section_at(RBinObject *o, ut64 off, int va) {
 	R_RETURN_VAL_IF_FAIL (o, NULL);
 	RBinSection *section;
 	RListIter *iter;
-	// TODO: must be O(1) .. use sdb here
+	// TODO: must be O(1) .. use memoization or tree or so
 	r_list_foreach (o->sections, iter, section) {
 		if (section->is_segment) {
 			continue;
@@ -790,15 +783,15 @@ R_API RList *r_bin_reset_strings(RBin *bin) {
 	ht_up_free (bf->bo->strings_db);
 	bf->bo->strings_db = ht_up_new0 ();
 
-	bf->rawstr = bin->rawstr;
+	bf->rawstr = bin->options.rawstr;
 	RBinPlugin *plugin = r_bin_file_cur_plugin (bf);
 
 	if (plugin && plugin->strings) {
 		bf->bo->strings = plugin->strings (bf);
 	} else {
-		bf->bo->strings = r_bin_file_get_strings (bf, bin->minstrlen, 0, bf->rawstr);
+		bf->bo->strings = r_bin_file_get_strings (bf, bin->options.minstrlen, 0, bf->rawstr);
 	}
-	if (bin->debase64) {
+	if (bin->options.debase64) {
 		r_bin_object_filter_strings (bf->bo);
 	}
 	return bf->bo->strings;
@@ -851,9 +844,6 @@ R_API RBin *r_bin_new(void) {
 	RBinXtrPlugin *static_xtr_plugin;
 	RBinLdrPlugin *static_ldr_plugin;
 	RBin *bin = R_NEW0 (RBin);
-	if (!bin) {
-		return NULL;
-	}
 	if (!r_str_constpool_init (&bin->constpool)) {
 		goto trashbin;
 	}
@@ -873,7 +863,7 @@ R_API RBin *r_bin_new(void) {
 	}
 	bin->cb_printf = (PrintfCallback)printf;
 	bin->plugins = r_list_newf ((RListFree)free);
-	bin->minstrlen = 0;
+	bin->options.minstrlen = 0;
 	bin->strpurge = NULL;
 	bin->strenc = NULL;
 	bin->want_dbginfo = true;
@@ -893,9 +883,6 @@ R_API RBin *r_bin_new(void) {
 		bin->binxtrs->free = free;
 		for (i = 0; bin_xtr_static_plugins[i]; i++) {
 			static_xtr_plugin = R_NEW0 (RBinXtrPlugin);
-			if (!static_xtr_plugin) {
-				goto trashbin_binxtrs;
-			}
 			*static_xtr_plugin = *bin_xtr_static_plugins[i];
 			if (r_bin_xtr_add (bin, static_xtr_plugin) == false) {
 				free (static_xtr_plugin);
@@ -908,9 +895,6 @@ R_API RBin *r_bin_new(void) {
 		bin->binldrs->free = free;
 		for (i = 0; bin_ldr_static_plugins[i]; i++) {
 			static_ldr_plugin = R_NEW0 (RBinLdrPlugin);
-			if (!static_ldr_plugin) {
-				goto trashbin_binldrs;
-			}
 			*static_ldr_plugin = *bin_ldr_static_plugins[i];
 			if (r_bin_ldr_add (bin, static_ldr_plugin) == false) {
 				free (static_ldr_plugin);
@@ -918,13 +902,13 @@ R_API RBin *r_bin_new(void) {
 		}
 	}
 	return bin;
-trashbin_binldrs:
+#if 0
 	r_list_free (bin->binldrs);
-trashbin_binxtrs:
 	r_list_free (bin->binxtrs);
 	r_list_free (bin->binfiles);
 	r_id_storage_free (bin->ids);
 	r_str_constpool_fini (&bin->constpool);
+#endif
 trashbin:
 	free (bin);
 	return NULL;
@@ -1046,7 +1030,7 @@ static char *get_arch_string(const char *arch, int bits, RBinInfo *info) {
 	return r_strbuf_drain (sb);
 }
 
-R_API void r_bin_list_archs(RBin *bin, PJ *pj, int mode) {
+R_API void r_bin_list_archs(RBin *bin, PJ *pj, RTable *t, int mode) {
 	R_RETURN_IF_FAIL (bin);
 
 	char unk[128];
@@ -1074,7 +1058,7 @@ R_API void r_bin_list_archs(RBin *bin, PJ *pj, int mode) {
 	if (!nbinfile) {
 		return;
 	}
-	RTable *table = r_table_new ("bins");
+	RTable *table = t? t: r_table_new ("bins");
 	const char *fmt = "dXnss";
 	if (mode == 'j') {
 		pj_ka (pj, "bins");
@@ -1245,7 +1229,7 @@ R_API RBuffer *r_bin_create(RBin *bin, const char *p,
 R_API RBuffer *r_bin_package(RBin *bin, const char *type, const char *file, RList *files) {
 	if (!strcmp (type, "zip")) {
 		// XXX: implement me
-		r_warn_if_reached ();
+		R_WARN_IF_REACHED ();
 	} else if (!strcmp (type, "fat")) {
 		// XXX: this should be implemented in the fat plugin, not here
 		// XXX should pick the callback from the plugin list
@@ -1380,19 +1364,17 @@ R_API void r_bin_load_filter(RBin *bin, ut64 rules) {
 }
 
 /* RBinField */
-R_API RBinField *r_bin_field_new(ut64 paddr, ut64 vaddr, ut64 value, int size, const char *name, R_NULLABLE const char *comment, R_NULLABLE const char *format, bool format_named) {
+R_API RBinField *r_bin_field_new(ut64 paddr, ut64 vaddr, ut64 value, int size, const char *name, const char * R_NULLABLE comment, const char * R_NULLABLE format, bool format_named) {
 	RBinField *ptr = R_NEW0 (RBinField);
-	if (ptr) {
-		ptr->name = r_bin_name_new (name);
-		ptr->comment = R_STR_ISNOTEMPTY (comment)? strdup (comment): NULL;
-		ptr->format = R_STR_ISNOTEMPTY (format)? strdup (format): NULL;
-		ptr->format_named = format_named;
-		ptr->vaddr = vaddr;
-		ptr->paddr = paddr;
-		ptr->size = size;
-		ptr->value = value;
-	//	ptr->attr = default attributes for fields?
-	}
+	ptr->name = r_bin_name_new (name);
+	ptr->comment = R_STR_ISNOTEMPTY (comment)? strdup (comment): NULL;
+	ptr->format = R_STR_ISNOTEMPTY (format)? strdup (format): NULL;
+	ptr->format_named = format_named;
+	ptr->vaddr = vaddr;
+	ptr->paddr = paddr;
+	ptr->size = size;
+	ptr->value = value;
+	// ptr->attr = default attributes for fields?
 	return ptr;
 }
 
@@ -1409,19 +1391,15 @@ R_API void r_bin_field_free(void *_field) {
 
 R_IPI RBinSection *r_bin_section_new(const char *name) {
 	RBinSection *s = R_NEW0 (RBinSection);
-	if (s) {
-		s->name = name? strdup (name): NULL;
-	}
+	s->name = name? strdup (name): NULL;
 	return s;
 }
 
 R_API RBinSection *r_bin_section_clone(RBinSection *s) {
 	RBinSection *d = R_NEW0 (RBinSection);
-	if (d) {
-		memcpy (d, s, sizeof (RBinSection));
-		d->name = s->name? strdup (s->name): NULL;
-		d->format = s->format? strdup (s->format): NULL;
-	}
+	memcpy (d, s, sizeof (RBinSection));
+	d->name = s->name? strdup (s->name): NULL;
+	d->format = s->format? strdup (s->format): NULL;
 	return d;
 }
 
@@ -1454,13 +1432,11 @@ R_API RBinFile *r_bin_file_at(RBin *bin, ut64 at) {
 
 R_API RBinTrycatch *r_bin_trycatch_new(ut64 source, ut64 from, ut64 to, ut64 handler, ut64 filter) {
 	RBinTrycatch *tc = R_NEW0 (RBinTrycatch);
-	if (tc) {
-		tc->source = source;
-		tc->from = from;
-		tc->to = to;
-		tc->handler = handler;
-		tc->filter = filter;
-	}
+	tc->source = source;
+	tc->from = from;
+	tc->to = to;
+	tc->handler = handler;
+	tc->filter = filter;
 	return tc;
 }
 
@@ -1483,18 +1459,14 @@ R_API const char *r_bin_field_kindstr(RBinField *f) {
 R_API RBinName *r_bin_name_new_from(R_OWN char *name) {
 	R_RETURN_VAL_IF_FAIL (name, NULL);
 	RBinName *bn = R_NEW0 (RBinName);
-	if (R_LIKELY (bn)) {
-		bn->oname = name;
-	}
+	bn->oname = name;
 	return bn;
 }
 
 R_API RBinName *r_bin_name_new(const char *name) {
 	R_RETURN_VAL_IF_FAIL (name, NULL);
 	RBinName *bn = R_NEW0 (RBinName);
-	if (R_LIKELY (bn)) {
-		bn->oname = strdup (name);
-	}
+	bn->oname = strdup (name);
 	return bn;
 }
 
@@ -1689,13 +1661,7 @@ R_API char *r_bin_attr_tostring(ut64 attr, bool singlechar) {
 	return r_strbuf_drain (sb);
 }
 
-// TODO : not implemented yet
-#if R2_USE_NEW_ABI
 R_API ut64 r_bin_attr_fromstring(const char *s, bool compact) {
-#else
-R_API ut64 r_bin_attr_fromstring(const char *s) {
-	const bool compact = false;
-#endif
 	size_t i;
 	ut64 bits = 0LL;
 	const char *word;
@@ -1706,7 +1672,7 @@ R_API ut64 r_bin_attr_fromstring(const char *s) {
 			for (i = 0; i < 64; i++) {
 				const char *bn = attr_bit_name (i, true);
 				if (bn && *w == *bn) {
-					bits |= (1 << i);
+					bits |= (1ULL << i);
 					break;
 				}
 			}
@@ -1730,7 +1696,7 @@ R_API ut64 r_bin_attr_fromstring(const char *s) {
 	return bits;
 }
 
-#if R2_USE_NEW_ABI
+// TODO: Must be a RBinFile.cmd() instead
 R_API bool r_bin_command(RBin *bin, const char *input) {
 	RBinFile *a = r_bin_cur (bin);
 	if (a) {
@@ -1741,4 +1707,3 @@ R_API bool r_bin_command(RBin *bin, const char *input) {
 	}
 	return false;
 }
-#endif
