@@ -23,13 +23,13 @@ static bool close_but_cb(void *user, void *data, ut32 id) {
 
 // TODO: move to IO as a helper?
 R_API bool r_core_file_close_all_but(RCore *core) {
-	r_id_storage_foreach (core->io->files, close_but_cb, core);
+	r_id_storage_foreach (&core->io->files, close_but_cb, core);
 	return true;
 }
 
 static bool its_a_mips(RCore *core) {
 	RArchConfig *cfg = core->rasm->config;
-	return cfg && cfg->arch && r_str_startswith (cfg->arch, "mips");
+	return cfg && r_str_startswith (cfg->arch, "mips");
 }
 
 static void load_gp(RCore *core) {
@@ -59,7 +59,7 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 	RBinFile *bf = odesc ? r_bin_file_find_by_fd (core->bin, odesc->fd) : NULL;
 	char *ofilepath = NULL, *obinfilepath = (bf && bf->file)? strdup (bf->file): NULL;
 	bool ret = false;
-	ut64 origoff = core->offset;
+	ut64 origoff = core->addr;
 	if (odesc) {
 		if (odesc->referer) {
 			ofilepath = odesc->referer;
@@ -78,7 +78,12 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 		}
 	}
 	if (new_baddr == UT64_MAX) {
-		new_baddr = r_config_get_i (core->config, "bin.baddr");
+		if (r_config_get_b (core->config, "bin.aslr")) {
+			new_baddr = ((ut64)r_num_rand (32)) << 24;
+			r_config_set_i (core->config, "bin.baddr", new_baddr);
+		} else {
+			new_baddr = r_config_get_i (core->config, "bin.baddr");
+		}
 	}
 
 	if (r_sandbox_enable (0)) {
@@ -158,7 +163,12 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 			} else if (new_baddr != UT64_MAX) {
 				baddr = new_baddr;
 			} else {
-				baddr = r_config_get_i (core->config, "bin.baddr");
+				if (r_config_get_b (core->config, "bin.aslr")) {
+					baddr = r_num_rand (32) << 24;
+					r_config_set_i (core->config, "bin.baddr", baddr);
+				} else {
+					baddr = r_config_get_i (core->config, "bin.baddr");
+				}
 			}
 			ret = r_core_bin_load (core, obinfilepath, baddr);
 			r_core_bin_update_arch_bits (core);
@@ -186,6 +196,8 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 	}
 	r_core_seek (core, origoff, true);
 	if (isdebug) {
+		r_core_cmd0 (core, "arp>$_"); // Fixes a bug where registers are not synced wtf
+		// r_debug_reg_sync (core->dbg, R_REG_TYPE_GPR, false); // does nothing
 		r_core_cmd0 (core, ".dm*");
 		r_core_cmd0 (core, ".dr*");
 		r_core_cmd_call (core, "sr PC");
@@ -258,10 +270,14 @@ R_API char *r_core_sysenv_begin(RCore *core, const char *cmd) {
 			}
 		}
 	}
-	r_sys_setenv ("R2PM_LEGACY", "0");
 	r_sys_setenv ("R2_COLOR", r_config_get (core->config, "scr.color"));
-	r_sys_setenv ("R2_OFFSET", r_strf ("%"PFMT64d, core->offset));
-	r_sys_setenv ("R2_XOFFSET", r_strf ("0x%08"PFMT64x, core->offset));
+
+	r_sys_setenv ("R2_ADDR", r_strf ("%"PFMT64d, core->addr));
+	r_sys_setenv ("R2_XADDR", r_strf ("0x%08"PFMT64x, core->addr));
+#if 0
+	r_sys_setenv ("R2_OFFSET", r_strf ("%"PFMT64d, core->addr));
+	r_sys_setenv ("R2_XOFFSET", r_strf ("0x%08"PFMT64x, core->addr));
+#endif
 	r_sys_setenv ("R2_BADDR", r_strf ("0x%08"PFMT64x, r_config_get_i (core->config, "bin.baddr")));
 	r_sys_setenv ("R2_ENDIAN", R_ARCH_CONFIG_IS_BIG_ENDIAN (core->rasm->config)? "big": "little");	// XXX
 	r_sys_setenv ("R2_BSIZE", r_strf ("%d", core->blocksize));
@@ -286,7 +302,6 @@ R_API char *r_core_sysenv_begin(RCore *core, const char *cmd) {
 	r_sys_setenv ("R2_COLOR", s);
 	free (s);
 	r_sys_setenv ("R2_UTF8", r_config_get_b (core->config, "scr.utf8")? "1": "0");
-	r_sys_setenv ("R2_DEBUG", r_config_get_b (core->config, "cfg.debug")? "1": "0");
 	r_sys_setenv ("R2_IOVA", r_config_get_b (core->config, "io.va")? "1": "0");
 #if 0
 	free (config_sdb_path);
@@ -348,7 +363,7 @@ static bool setbpint(RCore *r, const char *mode, const char *sym) {
 	if (!fi) {
 		return false;
 	}
-	bp = r_bp_add_sw (r->dbg->bp, fi->offset, 1, R_BP_PROT_EXEC);
+	bp = r_bp_add_sw (r->dbg->bp, fi->addr, 1, R_BP_PROT_EXEC);
 	if (bp) {
 		bp->internal = true;
 #if __linux__
@@ -364,7 +379,7 @@ static bool setbpint(RCore *r, const char *mode, const char *sym) {
 #endif
 
 // XXX - need to handle index selection during debugging
-static int r_core_file_load_for_debug(RCore *r, ut64 baseaddr, R_NULLABLE const char *filenameuri) {
+static int r_core_file_load_for_debug(RCore *r, ut64 baseaddr, const char * R_NULLABLE filenameuri) {
 	RIODesc *desc = r->io->desc;
 	RBinFile *bf = NULL;
 	RBinPlugin *plugin;
@@ -426,8 +441,8 @@ static int r_core_file_load_for_debug(RCore *r, ut64 baseaddr, R_NULLABLE const
 		// r_config_set_i (r->config, "io.va", false);
 		// r_config_set_b (r->config, "bin.str.raw", true);
 		// get bin.str.min
-		r->bin->minstrlen = r_config_get_i (r->config, "bin.str.min");
-		r->bin->maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
+		r->bin->options.minstrlen = r_config_get_i (r->config, "bin.str.min");
+		r->bin->options.maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
 	} else if (bf) {
 		RBinObject *obj = r_bin_cur_object (r->bin);
 		RBinInfo *info = obj? obj->info: NULL;
@@ -453,8 +468,15 @@ static int r_core_file_load_for_io_plugin(RCore *r, ut64 baseaddr, ut64 loadaddr
 	}
 	R_CRITICAL_ENTER (r);
 	r_io_use_fd (r->io, fd);
+	if (baseaddr == UT64_MAX) {
+		// ASLR - this is probably the only place where bin.aslr is used. maybe move into rbin.options before R2_600
+		if (r_config_get_b (r->config, "bin.aslr")) {
+			baseaddr = ((ut64)r_num_rand (32)) << 24;
+			r_config_set_i (r->config, "bin.baddr", baseaddr);
+		}
+	}
 	RBinFileOptions opt;
-	r_bin_file_options_init (&opt, fd, baseaddr, loadaddr, r->bin->rawstr);
+	r_bin_file_options_init (&opt, fd, baseaddr, loadaddr, r->bin->options.rawstr);
 	// opt.fd = fd;
 	opt.xtr_idx = xtr_idx;
 	if (!r_bin_open_io (r->bin, &opt)) {
@@ -495,8 +517,8 @@ static int r_core_file_load_for_io_plugin(RCore *r, ut64 baseaddr, ut64 loadaddr
 		// r_config_set_i (r->config, "io.va", false);
 		// r_config_set_b (r->config, "bin.str.raw", true);
 		// get bin.str.min
-		r->bin->minstrlen = r_config_get_i (r->config, "bin.str.min");
-		r->bin->maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
+		r->bin->options.minstrlen = r_config_get_i (r->config, "bin.str.min");
+		r->bin->options.maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
 	} else if (bf) {
 		RBinObject *obj = r_bin_cur_object (r->bin);
 		RBinInfo *info = obj? obj->info: NULL;
@@ -616,7 +638,7 @@ static bool filecb(void *user, void *data, ut32 id) {
 
 static bool file_is_loaded(RCore *core, const char *lib) {
 	MyFileData filedata = {lib, false};
-	r_id_storage_foreach (core->io->files, filecb, &filedata);
+	r_id_storage_foreach (&core->io->files, filecb, &filedata);
 	return filedata.found;
 }
 
@@ -645,6 +667,22 @@ static bool linkcb(void *user, void *data, ut32 id) {
 	return true;
 }
 
+static bool mustreopen(RCore *core, RIODesc *desc, const char *fn) {
+	if (r_str_startswith (fn, "stdio://")) {
+		return false;
+	}
+	if (r_str_startswith (fn, "nocache://")) {
+		return false;
+	}
+	if (r_str_startswith (fn, "mmap://")) {
+		return false;
+	}
+	if (fn && fn[0] != '-' && strcmp (fn, desc->uri)) {
+		return true;
+	}
+	return false;
+}
+
 R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 	R_RETURN_VAL_IF_FAIL (r && r->io, false);
 	R_CRITICAL_ENTER (r);
@@ -671,9 +709,12 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 			is_io_load = false;
 		}
 	}
-	r->bin->minstrlen = r_config_get_i (r->config, "bin.str.min");
-	r->bin->maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
+	r->bin->options.minstrlen = r_config_get_i (r->config, "bin.str.min");
+	r->bin->options.maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
 	R_CRITICAL_LEAVE (r);
+	RIODesc *odesc = NULL;
+	RIODesc *mustclose = NULL;
+				odesc = r->io->desc;
 	if (desc && is_io_load) {
 		int desc_fd = desc->fd;
 		// TODO? necessary to restore the desc back?
@@ -681,8 +722,11 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 		if ((desc->plugin && desc->plugin->isdbg) || r_config_get_b (r->config, "cfg.debug")) {
 			r_core_file_load_for_debug (r, baddr, filenameuri);
 		} else {
-			if (filenameuri && filenameuri[0] != '-' && strcmp (filenameuri, desc->uri)) {
+			if (mustreopen (r, desc, filenameuri)) {
 				r_core_file_open (r, filenameuri, 0, baddr);
+				if (odesc != r->io->desc) {
+					mustclose = r->io->desc;
+				}
 			}
 			r_core_file_load_for_io_plugin (r, baddr, 0LL);
 			desc = r->io->desc;
@@ -694,9 +738,10 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 	}
 	desc = r->io->desc;
 	RBinFile *binfile = r_bin_cur (r->bin);
-	if (r->bin->cur && r->bin->cur->bo && r->bin->cur->bo->plugin && r->bin->cur->bo->plugin->strfilter) {
+	RBinPlugin *bp = R_UNWRAP5 (r, bin, cur, bo, plugin);
+	if (bp) {
 		char msg[2];
-		msg[0] = r->bin->cur->bo->plugin->strfilter;
+		msg[0] = bp->strfilter;
 		msg[1] = 0;
 		r_config_set (r->config, "bin.str.filter", msg);
 	}
@@ -722,8 +767,8 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 			// r_config_set_b (r->config, "bin.str.raw", true);
 			// r_config_set_b (r->config, "io.va", false);
 			// get bin.str.min
-			r->bin->minstrlen = r_config_get_i (r->config, "bin.str.min");
-			r->bin->maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
+			r->bin->options.minstrlen = r_config_get_i (r->config, "bin.str.min");
+			r->bin->options.maxstrbuf = r_config_get_i (r->config, "bin.str.maxbuf");
 		} else if (binfile) {
 			RBinObject *obj = r_bin_cur_object (r->bin);
 			if (obj) {
@@ -793,15 +838,15 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 				// R_LOG_ERROR ("Cannot find '%s' import in the PLT", imp->name);
 				continue;
 			}
-			ut64 imp_addr = impsym->offset;
+			ut64 imp_addr = impsym->addr;
 			const char *imp_name = r_bin_name_tostring2 (imp->name, 'o');
 			eprintf ("Resolving %s... ", imp_name);
 			RCoreLinkData linkdata = {imp_name, UT64_MAX, r->bin};
-			r_id_storage_foreach (r->io->files, linkcb, &linkdata);
+			r_id_storage_foreach (&r->io->files, linkcb, &linkdata);
 			if (linkdata.addr != UT64_MAX) {
 				eprintf ("0x%08"PFMT64x, linkdata.addr);
-				ut64 a = linkdata.addr;
-				ut64 b = imp_addr;
+				const ut64 a = linkdata.addr;
+				const ut64 b = imp_addr;
 				r_core_cmdf (r, "ax 0x%08"PFMT64x" 0x%08"PFMT64x, a, b);
 			}
 		}
@@ -832,7 +877,7 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 				R_LOG_INFO ("Setting up coredump: Problem while setting the registers");
 			} else {
 				R_LOG_INFO ("Setting up coredump: Registers have been set");
-				const char *regname = r_reg_get_name (r->anal->reg, R_REG_NAME_SP);
+				const char *regname = r_reg_alias_getname (r->anal->reg, R_REG_ALIAS_SP);
 				if (regname) {
 					RRegItem *reg = r_reg_get (r->anal->reg, regname, -1);
 					if (reg) {
@@ -840,7 +885,7 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 						stack_map = r_io_map_get_at (r->io, sp_addr);
 					}
 				}
-				regname = r_reg_get_name (r->anal->reg, R_REG_NAME_PC);
+				regname = r_reg_alias_getname (r->anal->reg, R_REG_ALIAS_PC);
 				if (regname) {
 					RRegItem *reg = r_reg_get (r->anal->reg, regname, -1);
 					if (reg) {
@@ -873,6 +918,7 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 	}
 beach:
 	if (r_config_get_b (r->config, "bin.dbginfo") && R_STR_ISNOTEMPTY (filenameuri)) {
+		// TODO only for macho
 		// load companion dwarf files
 		const char *basename = r_file_basename (filenameuri);
 		char *macdwarf = r_str_newf ("%s.dSYM/Contents/Resources/DWARF/%s", filenameuri, basename);
@@ -885,7 +931,12 @@ beach:
 		}
 		free (macdwarf);
 	}
-
+	if (mustclose) {
+		r_io_desc_close (mustclose);
+		if (odesc) {
+			r_io_use_fd (r->io, odesc->fd);
+		}
+	}
 	r_flag_space_set (r->flags, "*");
 	return true;
 }
@@ -937,7 +988,7 @@ R_API RIODesc *r_core_file_open(RCore *r, const char *file, int flags, ut64 load
 	}
 	r->io->bits = r->rasm->config->bits; // TODO: we need an api for this
 	RIODesc *fd = r_io_open_nomap (r->io, file, flags, 0644);
-	if (r_cons_is_breaked()) {
+	if (r_cons_is_breaked (r->cons)) {
 		goto beach;
 	}
 	if (!fd) {
