@@ -31,6 +31,7 @@ static RCoreHelpMessage help_msg_c = {
 	"cc", " [at]", "compares in two hexdump columns of block size",
 	"ccc", " [at]", "same as above, but only showing different lines",
 	"ccd", " [at]", "compares in two disasm columns of block size",
+	"ccdf", " [at]", "compares function disasm with the other function",
 	"ccdd", " [at]", "compares decompiler output (e cmd.pdc=pdg|pdd)",
 	"cd", " [dir]", "chdir",
 	// "cc", " [offset]", "code bindiff current block against offset"
@@ -94,30 +95,21 @@ R_API R_BORROW RCoreCmpWatcher *r_core_cmpwatch_get(RCore *core, ut64 addr) {
 }
 
 R_API bool r_core_cmpwatch_add(RCore *core, ut64 addr, int size, const char *cmd) {
-	RCoreCmpWatcher *cmpw;
-	bool found = false;
-	R_RETURN_VAL_IF_FAIL (core && cmd && size > 0, false);
+	R_RETURN_VAL_IF_FAIL (core && size > 0, false);
 
-	cmpw = r_core_cmpwatch_get (core, addr);
+	bool found = false;
+	RCoreCmpWatcher *cmpw = r_core_cmpwatch_get (core, addr);
 	if (!cmpw) {
 		cmpw = R_NEW0 (RCoreCmpWatcher);
-		if (!cmpw) {
-			return false;
-		}
 		cmpw->addr = addr;
 	} else {
-		free (cmpw->odata);
-		free (cmpw->ndata);
-		free (cmpw->cmd);
+		R_FREE (cmpw->odata);
+		R_FREE (cmpw->ndata);
+		R_FREE (cmpw->cmd);
 		found = true;
 	}
 	cmpw->size = size;
-	cmpw->cmd = strdup (cmd);
-	if (!cmpw->cmd) {
-		free (cmpw);
-		return false;
-	}
-
+	cmpw->cmd = cmd? strdup (cmd): NULL;
 	cmpw->odata = NULL;
 	cmpw->ndata = malloc (size);
 	if (!cmpw->ndata) {
@@ -162,6 +154,23 @@ R_API bool r_core_cmpwatch_del(RCore *core, ut64 addr) {
 	return ret;
 }
 
+static char *cwcmd(RCore *core, RCoreCmpWatcher *w) {
+	if (!w->cmd) {
+		return NULL;
+	}
+	// use w->size for the temporary block size maybe?
+	if (strchr (w->cmd, '@')) {
+		if (strchr (w->cmd, ' ')) {
+			return r_core_cmd_strf (core, "%s", w->cmd);
+		}
+		return r_core_cmd_strf (core, "%s %d", w->cmd, w->size);
+	}
+	if (strchr (w->cmd, ' ')) {
+		return r_core_cmd_strf (core, "%s @ %"PFMT64d, w->cmd, w->addr);
+	}
+	return r_core_cmd_strf (core, "%s %d @ %" PFMT64d, w->cmd, w->size, w->addr);
+}
+
 R_API bool r_core_cmpwatch_show(RCore *core, ut64 addr, int mode) {
 	RListIter *iter;
 	RCoreCmpWatcher *w;
@@ -183,34 +192,64 @@ R_API bool r_core_cmpwatch_show(RCore *core, ut64 addr, int mode) {
 		}
 		switch (mode) {
 		case '*': // print watchers as r2 commands
-			r_cons_printf ("cw 0x%08" PFMT64x " %d %s%s\n",
-					w->addr, w->size, w->cmd,
+			r_cons_printf (core->cons, "cw 0x%08" PFMT64x " %d %s%s\n",
+					w->addr, w->size, r_str_get (w->cmd),
 					changed? " # differs": "");
 			break;
 		case 'q': // quiet
 			if (changed) {
-				r_cons_printf ("0x%08" PFMT64x " has changed\n", w->addr);
+				r_cons_printf (core->cons, "0x%08" PFMT64x " has changed\n", w->addr);
 			}
 			break;
 		case 'j': // "cw"
 			{
-			char *cmd_output = r_core_cmd_strf (core, "%s %d @%" PFMT64d,
-					w->cmd, w->size, w->addr);
 			pj_o (pj);
 			pj_kn (pj, "addr", w->addr);
 			pj_kb (pj, "changed", changed);
-			pj_ks (pj, "cmd", w->cmd);
-			pj_ks (pj, "cmd_out", r_str_get (cmd_output));
-			pj_end (pj);
+			if (w->cmd) {
+				pj_ks (pj, "cmd", w->cmd);
+			}
+			if (w->odata) {
+				char *m = r_hex_bin2strdup (w->odata, w->size);
+				pj_ks (pj, "oldmem", m);
+				free (m);
+			}
+			if (w->ndata) {
+				char *m = r_hex_bin2strdup (w->ndata, w->size);
+				pj_ks (pj, "newmem", m);
+				free (m);
+			}
+			char *cmd_output = cwcmd (core, w);
+			if (cmd_output) {
+				pj_ks (pj, "cmd_out", r_str_get (cmd_output));
+			}
 			free (cmd_output);
+			pj_end (pj);
 			}
 			break;
 		default:
-			r_cons_printf ("0x%08" PFMT64x "%s\n", w->addr, changed? " modified": "");
-			r_core_cmdf (core, "%s %d @%" PFMT64d, w->cmd, w->size, w->addr);
+			r_cons_printf (core->cons, "0x%08" PFMT64x "%s\n", w->addr, changed? " modified": "");
+			r_cons_print (core->cons, "  old: ");
+			int i;
+			if (w->odata) {
+				for (i = 0; i < w->size; i++) {
+					r_cons_printf (core->cons, "%02x", w->odata[i]);
+				}
+				r_cons_print (core->cons, " => new: ");
+			}
+			for (i = 0; i < w->size; i++) {
+				r_cons_printf (core->cons, "%02x", w->ndata[i]);
+			}
+			char *cmd_output = cwcmd (core, w);
+			if (cmd_output) {
+				r_cons_print (core->cons, "\n  cmd: ");
+				r_cons_println (core->cons, cmd_output);
+				free (cmd_output);
+			} else {
+				r_cons_newline (core->cons);
+			}
 			break;
 		}
-
 		ret = true;
 	}
 
@@ -218,7 +257,7 @@ R_API bool r_core_cmpwatch_show(RCore *core, ut64 addr, int mode) {
 		char *out;
 		pj_end (pj);
 		out = pj_drain (pj);
-		r_cons_println (out);
+		r_cons_println (core->cons, out);
 		free (out);
 	}
 
@@ -301,7 +340,7 @@ static int radare_compare_words(RCore *core, ut64 of, ut64 od, int len, int ws)
 	int i;
 	bool useColor = r_config_get_i (core->config, "scr.color") != 0;
 	utAny v0, v1;
-	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
+	RConsPrintablePalette *pal = &core->cons->context->pal;
 	for (i = 0; i < len; i+=ws) {
 		memset (&v0, 0, sizeof (v0));
 		memset (&v1, 0, sizeof (v1));
@@ -312,21 +351,21 @@ static int radare_compare_words(RCore *core, ut64 of, ut64 od, int len, int ws)
 		const char *colorEnd = useColor? Color_RESET: "";
 
 		if (useColor) {
-			r_cons_printf ("%s0x%08" PFMT64x"  "Color_RESET, pal->offset, of + i);
+			r_cons_printf (core->cons, "%s0x%08" PFMT64x"  "Color_RESET, pal->addr, of + i);
 		} else {
-			r_cons_printf ("0x%08" PFMT64x"  ", of + i);
+			r_cons_printf (core->cons, "0x%08" PFMT64x"  ", of + i);
 		}
 		switch (ws) {
 		case 1:
-			r_cons_printf ("%s0x%02x %c 0x%02x%s\n", color,
+			r_cons_printf (core->cons, "%s0x%02x %c 0x%02x%s\n", color,
 				(ut32)(v0.v8 & 0xff), ch, (ut32)(v1.v8 & 0xff), colorEnd);
 			break;
 		case 2:
-			r_cons_printf ("%s0x%04hx %c 0x%04hx%s\n", color,
+			r_cons_printf (core->cons, "%s0x%04hx %c 0x%04hx%s\n", color,
 				v0.v16, ch, v1.v16, colorEnd);
 			break;
 		case 4:
-			r_cons_printf ("%s0x%08"PFMT32x" %c 0x%08"PFMT32x"%s\n", color,
+			r_cons_printf (core->cons, "%s0x%08"PFMT32x" %c 0x%08"PFMT32x"%s\n", color,
 				v0.v32, ch, v1.v32, colorEnd);
 			//r_core_cmdf (core, "fd@0x%"PFMT64x, v0.v32);
 			if (v0.v32 != v1.v32) {
@@ -334,7 +373,7 @@ static int radare_compare_words(RCore *core, ut64 of, ut64 od, int len, int ws)
 			}
 			break;
 		case 8:
-			r_cons_printf ("%s0x%016"PFMT64x" %c 0x%016"PFMT64x"%s\n",
+			r_cons_printf (core->cons, "%s0x%016"PFMT64x" %c 0x%016"PFMT64x"%s\n",
 				color, v0.v64, ch, v1.v64, colorEnd);
 			//r_core_cmdf (core, "fd@0x%"PFMT64x, v0.v64);
 			if (v0.v64 != v1.v64) {
@@ -370,12 +409,12 @@ static int radare_compare_unified(RCore *core, ut64 of, ut64 od, int len) {
 	for (i = 0; i < len; i += inc) {
 		min = R_MIN (16, (len - i));
 		if (!memcmp (f + i, d + i, min)) {
-			r_cons_printf ("  ");
+			r_cons_printf (core->cons, "  ");
 			r_print_hexdiff (core->print, of + i, f + i, of + i, f + i, min, 0);
 		} else {
-			r_cons_printf ("- ");
+			r_cons_printf (core->cons, "- ");
 			r_print_hexdiff (core->print, of + i, f + i, od + i, d + i, min, 0);
-			r_cons_printf ("+ ");
+			r_cons_printf (core->cons, "+ ");
 			r_print_hexdiff (core->print, od + i, d + i, of + i, f + i, min, 0);
 		}
 	}
@@ -407,19 +446,19 @@ static int radare_compare(RCore *core, const ut8 *f, const ut8 *d, int len, int
 		}
 		switch (mode) {
 		case 0:
-			r_cons_printf ("0x%08"PFMT64x " (byte=%.2d)   %02x '%c'  ->  %02x '%c'\n",
-				core->offset + i, i + 1,
+			r_cons_printf (core->cons, "0x%08"PFMT64x " (byte=%.2d)   %02x '%c'  ->  %02x '%c'\n",
+				core->addr + i, i + 1,
 				f[i], (IS_PRINTABLE (f[i]))? f[i]: ' ',
 				d[i], (IS_PRINTABLE (d[i]))? d[i]: ' ');
 			break;
 		case '*':
-			r_cons_printf ("wx %02x @ 0x%08"PFMT64x "\n",
+			r_cons_printf (core->cons, "wx %02x @ 0x%08"PFMT64x "\n",
 				d[i],
-				core->offset + i);
+				core->addr + i);
 			break;
 		case 'j':
 			pj_o (pj);
-			pj_kn (pj, "offset", core->offset + i);
+			pj_kn (pj, "offset", core->addr + i);
 			pj_ki (pj, "rel_offset", i);
 			pj_ki (pj, "value", (int)f[i]);
 			pj_ki (pj, "cmp_value", (int)d[i]);
@@ -438,7 +477,7 @@ static int radare_compare(RCore *core, const ut8 *f, const ut8 *d, int len, int
 		pj_ki (pj, "total_bytes", len);
 		pj_end (pj); // End array
 		pj_end (pj); // End object
-		r_cons_println (pj_string (pj));
+		r_cons_println (core->cons, pj_string (pj));
 	}
 	return len - eq;
 }
@@ -455,77 +494,61 @@ static void nowatchers(ut64 addr) {
 static int cmd_cmp_watcher(RCore *core, const char *input) {
 	static RCoreHelpMessage help_msg_cw = {
 		"Usage: cw", "[args]", "Manage compare watchers; See if and how memory changes",
-		"cw??", "", "Show more info about watchers",
-		"cw ", "addr sz cmd", "Add a compare watcher",
+		"cw", "", "list comparison watchers",
+		"cw ", "addr", "Show comparison watcher for given address",
+		"cw ", "addr sz [(cmd)]", "Add a compare watcher (cmd is optional)",
 		"cw", "[*qj] [addr]", "Show compare watchers (*=r2 commands, q=quiet, j=json)",
 		"cwd", " [addr]", "Delete watcher",
 		"cwr", " [addr]", "Revert watcher",
 		"cwu", " [addr]", "Update watcher",
 		NULL
 	};
-	static const char *verbose_help_cw =
-		"Watchers are used to record memory at 2 different points in time, then\n"
-		"report if and how it changed. First, create one with `cw addr sz cmd`. This\n"
-		"will record sz bytes at addr. To record the second state, use `cwu`. Now, when\n"
-		"you run `cw`, the watcher will report if the bytes changed and run the command given\n"
-		"at creation with the size and address. You may overwrite any watcher by creating\n"
-		"another at the same address. This will discard the existing watcher completely.\n"
-		"\n"
-		"When you create a watcher, the data read from memory is marked as \"new\". Updating\n"
-		"the watcher with `cwu` will mark this data as \"old\", and then read the \"new\" data.\n"
-		"`cwr` will mark the current \"old\" state as being \"new\", letting you reuse it as\n"
-		"your new base state when updating with `cwu`. Any existing \"new\" state from running\n"
-		"`cwu` previously is lost in this process. Watched memory areas may overlap with no ill\n"
-		"effects, but may have unexpected results if you update some but not others.\n"
-		"\n"
-		"Showing a watcher without updating will still run the command, but it will not report\n"
-		"changes.\n"
-		"\n"
-		"When an address is an optional argument, the command will apply to all watchers if\n"
-		"you don't pass one.\n"
-		"\n"
-		"For more details and examples, see section 4.10 of the radare2 book.\n";
 
+	RCons *cons = core->cons;
 	ut64 addr = UT64_MAX;
 	int ret = 0;
 
 	switch (*input) {
 	case ' ': { // "cw "
-		int argc;
-		char **argv = r_str_argv (input + 1, &argc);
-		if (!argv) {
-			return 1;
-		}
-		if (argc == 1) { // "cw [addr]"
-			addr = r_num_math (core->num, argv[0]);
-			r_core_cmpwatch_show (core, addr, 0);
-		} else if (argc == 3) { // "cw addr sz cmd"
-			addr = r_num_math (core->num, argv[0]);
-			ut64 size = r_num_math (core->num, argv[1]);
-
-			if (size < 1) {
-				ret = 1;
-				R_LOG_ERROR ("Can't create a watcher with size less than 1");
-				goto out_free_argv;
-			}
-			if (size > INT_MAX) {
-				ret = 1;
-				R_LOG_ERROR ("Can't create a watcher with size larger than an int");
-				goto out_free_argv;
-			}
-
-			if (!r_core_cmpwatch_add (core, addr, (int)size, argv[2])) {
-				ret = 1;
-				R_LOG_ERROR ("Failed to add watcher");
+		char *arg_addr = r_str_trim_dup (input);
+		char *arg_size = strchr (arg_addr, ' ');
+		if (arg_size) {
+			*arg_size++ = 0;
+			r_str_trim (arg_size);
+			addr = r_num_math (core->num, arg_addr);
+			char *cmd = strchr (arg_size, ' ');
+			if (cmd) {
+				*cmd++ = 0;
+				st64 size = r_num_math (core->num, arg_size);
+				if (size < 1 || size > INT_MAX) {
+					ret = 1;
+					R_LOG_ERROR ("Invalid size");
+				} else if (!r_core_cmpwatch_add (core, addr, (int)size, cmd)) {
+					ret = 1;
+					R_LOG_ERROR ("Failed to add watcher");
+				}
+			} else {
+				st64 size = r_num_math (core->num, arg_size);
+				if (size < 1 || size > INT_MAX) {
+					ret = 1;
+					R_LOG_ERROR ("Invalid size");
+				} else if (!r_core_cmpwatch_add (core, addr, size, NULL)) {
+					ret = 1;
+					R_LOG_ERROR ("Failed to add watcher");
+				}
 			}
 		} else {
-			r_core_cmd_help_match (core, help_msg_cw, "cw ");
+			ut64 addr = r_num_math (core->num, arg_addr);
+			if (core->num->nc.errors != 0) {
+				addr = UT64_MAX;
+			}
+			r_core_cmpwatch_show (core, addr, 0);
 		}
-out_free_argv:
-		r_str_argv_free (argv);
+		free (arg_addr);
 		break;
 	}
-	case 'd': // "cwd"
+	case 'd': // "cwd" /
+	case '-': // "cwd" // TODO rename to cw-
 		if (input[1] == '?') {
 			r_core_cmd_help_match (core, help_msg_cw, "cwd");
 			return 0;
@@ -533,7 +556,7 @@ out_free_argv:
 		if (input[1]) {
 			addr = r_num_math (core->num, input + 2);
 		}
-		if (addr == UT64_MAX && !r_cons_yesno ('n', "Delete all watchers? (y/N)")) {
+		if (addr == UT64_MAX && !r_cons_yesno (cons, 'n', "Delete all watchers? (y/N)")) {
 			return 1;
 		}
 		if (!r_core_cmpwatch_del (core, addr) && addr) {
@@ -546,13 +569,10 @@ out_free_argv:
 			r_core_cmd_help_match (core, help_msg_cw, "cwr");
 			return 0;
 		}
-
 		if (input[1]) {
 			addr = r_num_math (core->num, input + 2);
 		}
-
-		if (addr == UT64_MAX &&
-				!r_cons_yesno ('n', "Revert all watchers? (y/N)")) {
+		if (addr == UT64_MAX && !r_cons_yesno (cons, 'n', "Revert all watchers? (y/N)")) {
 			return 1;
 		}
 
@@ -594,19 +614,45 @@ out_free_argv:
 	}
 	case '?': // "cw?"
 	default:
-		if (input[1] == '?') {
-			// this command really needs explaining
-			// so it's strongly signposted
-			r_cons_printf ("%s", verbose_help_cw);
-		} else {
-			r_core_cmd_help (core, help_msg_cw);
-		}
+		r_core_cmd_help (core, help_msg_cw);
 		break;
 	}
 
 	return ret;
 }
 
+static char *opstr(RCore *core, RAnalOp *op, ut64 addr) {
+	ut8 data[32];
+	char *str;
+	r_io_read_at (core->io, addr, data, sizeof (data));
+	// dis A
+	core->rasm->parse->subrel = r_config_get_b (core->config, "asm.sub.rel");
+	r_asm_set_pc (core->rasm, addr);
+	(void) r_asm_disassemble (core->rasm, op, data, sizeof (data));
+	char *opa = strdup (op->mnemonic);
+	if (r_config_get_b (core->config, "asm.sub.var")) {
+		str = r_asm_parse_subvar (core->rasm, NULL, addr, op->size, opa);
+		if (str) {
+			free (opa);
+			opa = str;
+		}
+	}
+	str = r_asm_parse_filter (core->rasm, addr, core->flags, NULL, opa);
+	if (str) {
+		free (opa);
+		opa = str;
+	}
+	if (r_config_get_b (core->config, "asm.imm.trim")) {
+		str = r_asm_parse_immtrim (core->rasm, opa);
+		if (str) {
+			free (opa);
+			opa = str;
+		}
+	}
+	// R_LOG_INFO (opa);
+	return opa;
+}
+
 static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 	RAnalOp op, op2;
 	int i, j;
@@ -615,7 +661,7 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 	int cols = r_config_get_i (core->config, "hex.cols") * 2;
 	ut64 off = r_num_math (core->num, input);
 	ut8 *buf = calloc (core->blocksize + 32, 1);
-	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
+	RConsPrintablePalette *pal = &core->cons->context->pal;
 	if (!buf) {
 		return false;
 	}
@@ -625,48 +671,41 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 		{
 #if 0
 		char *a = r_core_cmd_strf (core, "pdc @ 0x%"PFMT64x, off);
-		char *b = r_core_cmd_strf (core, "pdc @ 0x%"PFMT64x, core->offset);
+		char *b = r_core_cmd_strf (core, "pdc @ 0x%"PFMT64x, core->addr);
 		RDiff *d = r_diff_new ();
 		char *s = r_diff_buffers_unified (d, a, strlen(a), b, strlen(b));
-		r_cons_printf ("%s\n", s);
+		r_cons_printf (core->cons, "%s\n", s);
 		free (a);
 		free (b);
 		free (s);
 		r_diff_free (d);
 #else
 		r_core_cmdf (core, "pdc @ 0x%"PFMT64x">$a", off);
-		r_core_cmdf (core, "pdc @ 0x%"PFMT64x">$b", core->offset);
+		r_core_cmdf (core, "pdc @ 0x%"PFMT64x">$b", core->addr);
 		r_core_cmd0 (core, "diff $a $b;rm $a;rm $b");
 #endif
 		}
 		break;
 	case 'c': // columns
 		for (i = j = 0; i < core->blocksize && j < core->blocksize;) {
-			// dis A
-			r_asm_set_pc (core->rasm, core->offset + i);
-			(void) r_asm_disassemble (core->rasm, &op,
-				core->block + i, core->blocksize - i);
-
-			// dis B
-			r_asm_set_pc (core->rasm, off + i);
-			(void) r_asm_disassemble (core->rasm, &op2,
-				buf + j, core->blocksize - j);
+			char *opa = opstr (core, &op, core->addr + i);
+			char *opb = opstr (core, &op2, off + i);
 
 			// show output
-			bool iseq = !strcmp (op.mnemonic, op2.mnemonic);
+			bool iseq = !strcmp (opa, opb); // op.mnemonic, op2.mnemonic);
 			memset (colpad, ' ', sizeof (colpad));
 			{
-				int pos = strlen (op.mnemonic);
+				int pos = strlen (opa); // op.mnemonic);
 				pos = (pos > cols)? 0: cols - pos;
 				colpad[pos] = 0;
 			}
 			if (hascolor) {
-				r_cons_print (iseq? pal->graph_true: pal->graph_false);
+				r_cons_print (core->cons, iseq? pal->graph_true: pal->graph_false);
 			}
-			r_cons_printf (" 0x%08"PFMT64x "  %s %s", core->offset + i, op.mnemonic, colpad);
-			r_cons_printf ("%c 0x%08"PFMT64x "  %s\n", iseq? '=': '!', off + j, op2.mnemonic);
+			r_cons_printf (core->cons, " 0x%08"PFMT64x "  %s %s", core->addr + i, opa, colpad);
+			r_cons_printf (core->cons, "%c 0x%08"PFMT64x "  %s\n", iseq? '=': '!', off + j, opb);
 			if (hascolor) {
-				r_cons_print (Color_RESET);
+				r_cons_print (core->cons, Color_RESET);
 			}
 			if (op.size < 1) {
 				op.size = 1;
@@ -676,12 +715,14 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 				op2.size = 1;
 			}
 			j += op2.size;
+			free (opa);
+			free (opb);
 		}
 		break;
 	case 'u': // unified
 		for (i = j = 0; i < core->blocksize && j < core->blocksize;) {
 			// dis A
-			r_asm_set_pc (core->rasm, core->offset + i);
+			r_asm_set_pc (core->rasm, core->addr + i);
 			(void) r_asm_disassemble (core->rasm, &op,
 				core->block + i, core->blocksize - i);
 
@@ -693,19 +734,19 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 			// show output
 			bool iseq = !strcmp (op.mnemonic, op2.mnemonic);
 			if (iseq) {
-				r_cons_printf (" 0x%08"PFMT64x "  %s\n",
-					core->offset + i, op.mnemonic);
+				r_cons_printf (core->cons, " 0x%08"PFMT64x "  %s\n",
+					core->addr + i, op.mnemonic);
 			} else {
 				if (hascolor) {
-					r_cons_print (pal->graph_false);
+					r_cons_print (core->cons, pal->graph_false);
 				}
-				r_cons_printf ("-0x%08"PFMT64x "  %s\n", core->offset + i, op.mnemonic);
+				r_cons_printf (core->cons, "-0x%08"PFMT64x "  %s\n", core->addr + i, op.mnemonic);
 				if (hascolor) {
-					r_cons_print (pal->graph_true);
+					r_cons_print (core->cons, pal->graph_true);
 				}
-				r_cons_printf ("+0x%08"PFMT64x "  %s\n", off + j, op2.mnemonic);
+				r_cons_printf (core->cons, "+0x%08"PFMT64x "  %s\n", off + j, op2.mnemonic);
 				if (hascolor) {
-					r_cons_print (Color_RESET);
+					r_cons_print (core->cons, Color_RESET);
 				}
 			}
 			if (op.size < 1) {
@@ -791,7 +832,7 @@ static int cmd_cp(void *data, const char *input) {
  * colored graph_false.
  */
 static int cmp_bits(RCore *core, ut64 addr) {
-	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
+	RConsPrintablePalette *pal = &core->cons->context->pal;
 	const bool use_color = r_config_get_b (core->config, "scr.color");
 	const char *color_end = use_color? Color_RESET: "";
 	int i;
@@ -799,16 +840,16 @@ static int cmp_bits(RCore *core, ut64 addr) {
 	bool a_bits[8], b_bits[8];
 	const char *a_colors[8], *b_colors[8];
 
-	r_io_nread_at (core->io, core->offset, &a, 1);
+	r_io_nread_at (core->io, core->addr, &a, 1);
 	r_io_nread_at (core->io, addr, &b, 1);
 
 	/* Print offset header if enabled */
 	if (r_config_get_i (core->config, "hex.header")) {
-		const char *color = use_color? pal->offset: "";
-		char *n = r_str_newf ("0x%08" PFMT64x, core->offset);
+		const char *color = use_color? pal->addr: "";
+		char *n = r_str_newf ("0x%08" PFMT64x, core->addr);
 		const char *padding = r_str_pad (' ', strlen (n) - 10);
 		free (n);
-		r_cons_printf ("%s- offset -%s  7 6 5 4 3 2 1 0%s\n", color, padding, color_end);
+		r_cons_printf (core->cons, "%s- offset -%s  7 6 5 4 3 2 1 0%s\n", color, padding, color_end);
 	}
 
 	/* Set up bits and colors */
@@ -825,16 +866,16 @@ static int cmp_bits(RCore *core, ut64 addr) {
 		}
 	}
 
-	r_cons_printf ("%s0x%08" PFMT64x "%s  ", use_color? pal->graph_false: "", core->offset, color_end);
+	r_cons_printf (core->cons, "%s0x%08" PFMT64x "%s  ", use_color? pal->graph_false: "", core->addr, color_end);
 	for (i = 7; i >= 0; i--) {
-		r_cons_printf ("%s%d%s%s", a_colors[i], a_bits[i], color_end, i? " ": "");
+		r_cons_printf (core->cons, "%s%d%s%s", a_colors[i], a_bits[i], color_end, i? " ": "");
 	}
 
-	r_cons_printf ("\n%s0x%08" PFMT64x "%s  ", use_color? pal->graph_true: "", addr, color_end);
+	r_cons_printf (core->cons, "\n%s0x%08" PFMT64x "%s  ", use_color? pal->graph_true: "", addr, color_end);
 	for (i = 7; i >= 0; i--) {
-		r_cons_printf ("%s%d%s%s", b_colors[i], b_bits[i], color_end, i? " ": "");
+		r_cons_printf (core->cons, "%s%d%s%s", b_colors[i], b_bits[i], color_end, i? " ": "");
 	}
-	r_cons_newline ();
+	r_cons_newline (core->cons);
 
 	// 0 if equal, 1 if not equal
 	// same return pattern as ?==
@@ -913,7 +954,7 @@ static void _core_cmp_info_libs(RCore *core, int id0, int id1) {
 				found = true;
 			}
 		}
-		r_cons_printf ("%s%s\n", found? " ": "-", s);
+		r_cons_printf (core->cons, "%s%s\n", found? " ": "-", s);
 	}
 	r_list_foreach (s1, iter, s) {
 		bool found = false;
@@ -923,7 +964,7 @@ static void _core_cmp_info_libs(RCore *core, int id0, int id1) {
 			}
 		}
 		if (!found) {
-			r_cons_printf ("+%s\n", s);
+			r_cons_printf (core->cons, "+%s\n", s);
 		}
 	}
 	// r_list_free (s0);
@@ -952,7 +993,7 @@ static void _core_cmp_info_imports(RCore *core, int id0, int id1) {
 				found = true;
 			}
 		}
-		r_cons_printf ("%s%s\n", found? " ": "-", s_name);
+		r_cons_printf (core->cons, "%s%s\n", found? " ": "-", s_name);
 	}
 	r_list_foreach (s1, iter, s) {
 		const char *s_name = r_bin_name_tostring (s->name);
@@ -964,7 +1005,7 @@ static void _core_cmp_info_imports(RCore *core, int id0, int id1) {
 			}
 		}
 		if (!found) {
-			r_cons_printf ("+%s\n", s_name);
+			r_cons_printf (core->cons, "+%s\n", s_name);
 		}
 	}
 	// r_list_free (s0);
@@ -993,7 +1034,7 @@ static void _core_cmp_info_exports(RCore *core, int id0, int id1) {
 				found = true;
 			}
 		}
-		r_cons_printf ("%s%s\n", found? " ": "-", s_name);
+		r_cons_printf (core->cons, "%s%s\n", found? " ": "-", s_name);
 	}
 	r_list_foreach (s1, iter, s) {
 		const char *s_name = r_bin_name_tostring (s->name);
@@ -1005,7 +1046,7 @@ static void _core_cmp_info_exports(RCore *core, int id0, int id1) {
 			}
 		}
 		if (!found) {
-			r_cons_printf ("+%s\n", s_name);
+			r_cons_printf (core->cons, "+%s\n", s_name);
 		}
 	}
 	r_list_free (s0);
@@ -1034,7 +1075,7 @@ static void _core_cmp_info_symbols(RCore *core, int id0, int id1) {
 				found = true;
 			}
 		}
-		r_cons_printf ("%s%s\n", found? " ": "-", sname);
+		r_cons_printf (core->cons, "%s%s\n", found? " ": "-", sname);
 	}
 	r_list_foreach (s1, iter, s) {
 		bool found = false;
@@ -1046,7 +1087,7 @@ static void _core_cmp_info_symbols(RCore *core, int id0, int id1) {
 			}
 		}
 		if (!found) {
-			r_cons_printf ("+%s\n", sname);
+			r_cons_printf (core->cons, "+%s\n", sname);
 		}
 	}
 }
@@ -1098,12 +1139,12 @@ static void cmd_curl(RCore *core, const char *arg) {
 		if (r_str_startswith (arg, "http://") || r_str_startswith (arg, "https://")) {
 			int len;
 			char *s = postdata
-				? r_socket_http_post (arg, postdata, NULL, &len)
-				: r_socket_http_get (arg, NULL, &len);
+				? r_socket_http_post (arg, NULL, postdata, NULL, &len)
+				: r_socket_http_get (arg, NULL, NULL, &len);
 			if (s) {
-				r_cons_write (s, len);
+				r_cons_write (core->cons, s, len);
 				free (s);
-				r_cons_newline ();
+				r_cons_newline (core->cons);
 			}
 		} else {
 			r_core_cmd_help_match (core, help_msg_cu, "curl");
@@ -1150,8 +1191,8 @@ static int cmd_cmp_posix(RCore *core, const char *a, const char *b) {
 	return res;
 }
 
+static R_TH_LOCAL char *oldcwd = NULL;
 static int cmd_cmp(void *data, const char *input) {
-	static R_TH_LOCAL char *oldcwd = NULL;
 	int ret = 0, i, mode = 0;
 	RCore *core = (RCore *)data;
 	ut64 val = UT64_MAX;
@@ -1171,7 +1212,7 @@ static int cmd_cmp(void *data, const char *input) {
 			if (*path == '$') {
 				char *v_str = r_core_slurp (core, path, NULL);
 				if (v_str) {
-					r_cons_println (v_str);
+					r_cons_println (core->cons, v_str);
 					free (v_str);
 				}
 			} else if (*path) {
@@ -1180,7 +1221,7 @@ static int cmd_cmp(void *data, const char *input) {
 				} else {
 					char *res = r_syscmd_cat (path);
 					if (res) {
-						r_cons_print (res);
+						r_cons_print (core->cons, res);
 						free (res);
 					}
 				}
@@ -1384,19 +1425,36 @@ static int cmd_cmp(void *data, const char *input) {
 		if (input[1] == '?') { // "cc?"
 			r_core_cmd_help_contains (core, help_msg_c, "cc");
 		} else if (input[1] == 'd') { // "ccd"
-			if (input[2] == 'd') { // "ccdd"
+			if (input[2] == '?') {
+				r_core_cmd_help_contains (core, help_msg_c, "ccd");
+			} else if (input[2] == 'd') { // "ccdd"
 				cmd_cmp_disasm (core, input + 3, 'd');
 			} else {
 				cmd_cmp_disasm (core, input + 2, 'c');
 			}
-		} else {
+		} else if (input[1] == 'f') { // "ccdf"
+			RAnalFunction *fcn = r_anal_get_function_at (core->anal, core->addr);
+			if (fcn) {
+				const int obs = core->blocksize;
+				const int fsz = r_anal_function_linear_size (fcn);
+				r_core_block_size (core, fsz);
+				if (input[2] == ' ') {
+					r_core_cmdf (core, "ccd%s", input + 2);
+				} else {
+					R_LOG_ERROR ("Missing argument");
+				}
+				r_core_block_size (core, obs);
+			} else {
+				R_LOG_ERROR ("Cannot find function");
+			}
+		} else if (!input[1] || input[1] == ' ') {
 			ut32 oflags = core->print->flags;
 			ut64 addr = 0; // TOTHINK: Not sure what default address should be
 			if (input[1] == 'c') { // "ccc"
 				core->print->flags |= R_PRINT_FLAGS_DIFFOUT;
 				addr = r_num_math (core->num, input + 2);
 			} else {
-				if (*input && input[1]) {
+				if (input[0] && input[1]) {
 					addr = r_num_math (core->num, input + 2);
 				}
 			}
@@ -1405,11 +1463,13 @@ static int cmd_cmp(void *data, const char *input) {
 			if (b) {
 				memset (b, 0xff, core->blocksize);
 				r_io_read_at (core->io, addr, b, core->blocksize);
-				r_print_hexdiff (core->print, core->offset, block,
+				r_print_hexdiff (core->print, core->addr, block,
 					addr, b, core->blocksize, col);
 				free (b);
 			}
 			core->print->flags = oflags;
+		} else {
+			r_core_return_invalid_command (core, "cc", input[1]);
 		}
 		break;
 	case 'i': // "ci"
@@ -1438,11 +1498,11 @@ static int cmd_cmp(void *data, const char *input) {
 				RAnalFunction *fcn;
 				r_list_foreach (core->anal->fcns, iter, fcn) {
 					// R_LOG_INFO ("compare %s", fcn->name);
-					r_core_gdiff_fcn (core, core->offset, fcn->addr);
+					r_core_gdiff_fcn (core, core->addr, fcn->addr);
 				}
 			} else {
 				r_anal_diff_setup (core->anal, true, -1, -1);
-				r_core_gdiff_fcn (core, core->offset,
+				r_core_gdiff_fcn (core, core->addr,
 					r_num_math (core->num, input + 2));
 			}
 			return false;
@@ -1483,8 +1543,9 @@ static int cmd_cmp(void *data, const char *input) {
 		r_core_diff_show (core, core2);
 		/* exchange a segfault with a memleak */
 		core2->config = NULL;
-		r_core_free (core2);
+	//	r_core_free (core2);
 		r_core_bind_cons (core);
+r_cons_global (core->cons);
 	}
 	break;
 	case 'u': // "cu"
@@ -1496,7 +1557,7 @@ static int cmd_cmp(void *data, const char *input) {
 		switch (input[1]) {
 		case '.':
 		case ' ':
-			radare_compare_unified (core, core->offset,
+			radare_compare_unified (core, core->addr,
 				r_num_math (core->num, input + 2),
 				core->blocksize);
 			break;
@@ -1504,7 +1565,7 @@ static int cmd_cmp(void *data, const char *input) {
 		case '2':
 		case '4':
 		case '8':
-			radare_compare_words (core, core->offset,
+			radare_compare_words (core, core->addr,
 				r_num_math (core->num, input + 2),
 				core->blocksize, input[1] - '0');
 			break;
@@ -1535,7 +1596,7 @@ static int cmd_cmp(void *data, const char *input) {
 		case '1': { // "cv1"
 			ut8 n = (ut8) r_num_math (core->num, input + 2);
 			if (block[0] == n) {
-				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				r_cons_printf (core->cons, "0x%08"PFMT64x "\n", core->addr);
 				r_core_return_value (core, 0);
 			} else {
 				r_core_return_value (core, 1);
@@ -1545,7 +1606,7 @@ static int cmd_cmp(void *data, const char *input) {
 		case '2': { // "cv2"
 			ut16 n = (ut16) r_num_math (core->num, input + 2);
 			if (core->blocksize >= 2 && *(ut16*)block == n) {
-				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				r_cons_printf (core->cons, "0x%08"PFMT64x "\n", core->addr);
 				r_core_return_value (core, 0);
 			} else {
 				r_core_return_value (core, 1);
@@ -1555,7 +1616,7 @@ static int cmd_cmp(void *data, const char *input) {
 		case '4': { // "cv4"
 			ut32 n = (ut32) r_num_math (core->num, input + 2);
 			if (core->blocksize >= 4 && *(ut32*)block == n) {
-				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				r_cons_printf (core->cons, "0x%08"PFMT64x "\n", core->addr);
 				r_core_return_value (core, 0);
 			} else {
 				r_core_return_value (core, 1);
@@ -1565,7 +1626,7 @@ static int cmd_cmp(void *data, const char *input) {
 		case '8': { // "cv8"
 			ut64 n = (ut64) r_num_math (core->num, input + 2);
 			if (core->blocksize >= 8 && *(ut64*)block == n) {
-				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				r_cons_printf (core->cons, "0x%08"PFMT64x "\n", core->addr);
 				r_core_return_value (core, 0);
 			} else {
 				r_core_return_value (core, 1);
@@ -1607,14 +1668,16 @@ static int cmd_cmp(void *data, const char *input) {
 		}
 		break;
 	}
+	case 'L': // "cL"
+		r_core_cmdf (core, "Lc%s", input + 1);
+		break;
 	case 'l': // "cl"
 		if (strchr (input, 'f')) {
-			r_cons_flush ();
+			r_cons_flush (core->cons);
 		} else if (input[1] == 0) {
-			r_cons_fill_line ();
-			// r_cons_clear_line (0);
+			r_cons_fill_line (core->cons);
 		} else if (!strchr (input, '0')) {
-			r_cons_clear00 ();
+			r_cons_clear00 (core->cons);
 		}
 		break;
 	case 'm': // "cmp"
