@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2021-2024 - pancake */
+/* radare - LGPL - Copyright 2021-2025 - pancake */
 
 #define R_LOG_ORIGIN "r2pm"
 
@@ -24,9 +24,11 @@ static const char *helpmsg =
 	" -H ([variable])   list all or selected r2pm environment variables\n"
 	" -i <pkgname>      install/update package and its dependencies (see -c, -g)\n"
 	" -I                information about the repository and installed packages\n"
+	" -j                json output\n"
 	" -l                list installed packages\n"
 	" -q                be quiet\n"
 	" -r [cmd ...args]  run shell command with R2PM_BINDIR in PATH\n"
+	" -R <pkgname>      reload plugin (See R2PM_RELOAD code block package)\n"
 	" -s [<keyword>]    search available packages in database matching a string\n"
 	" -t [YYYY-MM-DD]   set a moment in time to pull the code from the git packages\n"
 	" -u <pkgname>      uninstall package (see -f to force uninstall)\n"
@@ -51,12 +53,13 @@ typedef struct r_r2pm_t {
 	bool list;
 	bool plugdir; // requires -c/clean
 	bool quiet;
+	bool json;
 	bool run;
 	bool search;
+	bool reload;
 	bool uninstall;
 	bool upgrade;
 	bool version;
-
 	int rc;
 	const char *time;
 } R2Pm;
@@ -110,10 +113,6 @@ static bool r2pm_add(R2Pm *r2pm, const char *repository) {
 	return false;
 }
 
-static char *r2pm_bindir(void) {
-	return r_xdg_datadir ("prefix/bin");
-}
-
 static char *r2pm_gitdir(void) {
 	char *e = r_sys_getenv ("R2PM_GITDIR");
 	if (R_STR_ISNOTEMPTY (e)) {
@@ -191,10 +190,7 @@ static char *find_newline(char *s) {
 	if (r && n) {
 		return (r < n)? r: n;
 	}
-	if (r) {
-		return r;
-	}
-	return n;
+	return r? r: n;
 
 }
 static char *r2pm_get(const char *file, const char *token, R2pmTokenType type) {
@@ -299,22 +295,37 @@ static char *r2pm_desc(const char *file) {
 	return r2pm_get (file, "\nR2PM_DESC ", TT_TEXTLINE);
 }
 
-static char *r2pm_list(void) {
+static char *r2pm_list(int mode) {
 	char *path = r2pm_pkgdir ();
 	RList *files = r_sys_dir (path);
 	free (path);
 	if (!files) {
 		return NULL;
 	}
-	RStrBuf *sb = r_strbuf_new ("");
+	PJ *pj = NULL;
+	RStrBuf *sb = NULL;
+	if (mode == 'j') {
+		pj = pj_new ();
+		pj_a (pj);
+	} else {
+		sb = r_strbuf_new ("");
+	}
 	RListIter *iter;
 	const char *file;
 	r_list_foreach (files, iter, file) {
 		if (*file != '.') {
-			r_strbuf_appendf (sb, "%s\n", file);
+			if (pj) {
+				pj_s (pj, file);
+			} else {
+				r_strbuf_appendf (sb, "%s\n", file);
+			}
 		}
 	}
 	r_list_free (files);
+	if (pj) {
+		pj_end (pj);
+		return pj_drain (pj);
+	}
 	return r_strbuf_drain (sb);
 }
 
@@ -350,6 +361,25 @@ static int r2pm_update(bool force) {
 	return rc;
 }
 
+// set python virtual environment when available
+static void r2pm_set_pyvenv(R2Pm *r2pm) {
+	char *r2_prefix = r_xdg_datadir ("prefix");
+	if (!r2_prefix) {
+		R_LOG_ERROR ("Cannot resolve xdg.datadir('prefix')");
+		return;
+	}
+	// set PYTHOMHOME, VIRTUAL_ENV and PATH
+	char *venv = r_file_new (r2_prefix, "venv", NULL);
+	if (r_file_is_directory (venv)) {
+		r_sys_setenv ("VIRTUAL_ENV", venv);
+		char *venv_bin = r_file_new (venv, "bin", NULL);
+		r_sys_setenv_sep ("PATH", venv_bin, false);
+		free (venv_bin);
+	}
+	free (r2_prefix);
+	free (venv);
+}
+
 static void r2pm_setenv(R2Pm *r2pm) {
 	char *gmake = r_file_path ("gmake");
 	if (gmake) {
@@ -358,6 +388,7 @@ static void r2pm_setenv(R2Pm *r2pm) {
 		r_sys_setenv ("MAKE", "make");
 	}
 	free (gmake);
+	r2pm_set_pyvenv (r2pm);
 
 	if (r2pm->global) {
 		// the r2pm_plugdir changes when using -g
@@ -424,28 +455,19 @@ static void r2pm_setenv(R2Pm *r2pm) {
 	free (r2pm_pkgcfg);
 	free (pkgcfg);
 
-	char *bindir = r_str_newf ("%s/bin", r2_prefix);
-	r_sys_setenv ("R2PM_BINDIR", bindir);
-	free (bindir);
+	char *r2pm_bindir = r_str_newf ("%s/bin", r2_prefix);
+	r_sys_mkdirp (r2pm_bindir);
+	r_sys_setenv ("R2PM_BINDIR", r2pm_bindir);
+	r_sys_setenv_sep ("PATH", r2pm_bindir, false);
+	r_sys_setenv_sep ("PATH", R2_BINDIR, false);
+	free (r2pm_bindir);
 
-	char *libdir = r_str_newf ("%s/lib", r2_prefix);
-	r_sys_setenv ("R2PM_LIBDIR", libdir);
-	free (libdir);
+	char *mandir = r_str_newf ("%s/man", r2_prefix);
+	r_sys_setenv("R2PM_MANDIR", mandir);
+	free (mandir);
 
-	char *incdir = r_str_newf ("%s/include", r2_prefix);
-	r_sys_setenv ("R2PM_INCDIR", incdir);
-	free (incdir);
-
-	char *oldpath = r_sys_getenv ("PATH");
-	if (!oldpath) {
-		oldpath = strdup ("/bin");
-	}
-	if (!strstr (oldpath, r2_prefix)) {
-		char *newpath = r_str_newf ("%s/bin%s%s", r2_prefix, R_SYS_ENVSEP, oldpath);
-		r_sys_setenv ("PATH", newpath);
-		free (newpath);
-	}
-	free (oldpath);
+	char *r2pm_libdir = r_str_newf ("%s/lib", r2_prefix);
+	r_sys_setenv ("R2PM_LIBDIR", r2pm_libdir);
 #if R2__WINDOWS__
 	const char *ldpathvar = NULL;
 #elif __HAIKU__
@@ -455,47 +477,14 @@ static void r2pm_setenv(R2Pm *r2pm) {
 #else
 	const char *ldpathvar = "LD_LIBRARY_PATH";
 #endif
-	char *opath = r_sys_getenv ("PATH");
-	if (opath) {
-		char *bindir = r2pm_bindir ();
-		r_sys_mkdirp (bindir);
-		const char *sep = R_SYS_ENVSEP;
-		char *newpath = r_str_newf ("%s%s%s", bindir, sep, opath);
-		r_sys_setenv ("PATH", newpath);
-		free (newpath);
-		free (opath);
-		free (bindir);
-	}
+	r_sys_setenv_sep (ldpathvar, r2pm_libdir, false);
+	r_sys_setenv_sep (ldpathvar, R2_LIBDIR, false);
+	free (r2pm_libdir);
 
-	char *ldpath = r_sys_getenv (ldpathvar);
-	if (!ldpath) {
-		ldpath = strdup ("");
-	}
-	if (!strstr (ldpath, r2_prefix)) {
-		char *newpath = r_str_newf ("%s/lib%s%s", r2_prefix, R_SYS_ENVSEP, ldpath);
-		r_sys_setenv (ldpathvar, newpath);
-		free (ldpath);
-		ldpath = newpath;
-	}
-	char *gr2_prefix = r_sys_cmd_str ("radare2 -NN -H R2_PREFIX", NULL, NULL);
-	if (gr2_prefix) {
-		r_str_trim (gr2_prefix);
-		if (R_STR_ISNOTEMPTY (gr2_prefix)) {
-			if (!strstr (ldpath, gr2_prefix)) {
-				char *newpath = r_str_newf ("%s/lib%s%s", gr2_prefix, R_SYS_ENVSEP, ldpath);
-				r_sys_setenv (ldpathvar, newpath);
-				free (newpath);
-			}
-		}
-		free (gr2_prefix);
-	}
+	char *incdir = r_str_newf ("%s/include", r2_prefix);
+	r_sys_setenv ("R2PM_INCDIR", incdir);
+	free (incdir);
 
-	if (!strstr (ldpath, r2_prefix)) {
-		char *newpath = r_str_newf ("%s/lib%s%s", r2_prefix, R_SYS_ENVSEP, ldpath);
-		r_sys_setenv (ldpathvar, newpath);
-		free (newpath);
-	}
-	free (ldpath);
 	free (r2_prefix);
 	// GLOBAL = 0 # depends on r2pm.global, which is set on r2pm_install
 	static const char *python_bins[] = {
@@ -649,6 +638,16 @@ static int r2pm_clone(const char *pkg) {
 	char *srcdir = r_file_new (pkgdir, pkg, NULL);
 	free (pkgdir);
 
+#if R2__WINDOWS__
+	char *script = r2pm_get (pkg, "\nR2PM_INSTALL_WINDOWS() {\n", TT_CODEBLOCK);
+	if (!script) {
+		R_LOG_ERROR ("This package does not have R2PM_INSTALL_WINDOWS instructions");
+		free (srcdir);
+		return 1;
+	}
+	free (script);
+#endif
+
 	bool offline = r_sys_getenv_asbool ("R2PM_OFFLINE");
 	if (offline) {
 		free (srcdir);
@@ -748,11 +747,16 @@ static int r2pm_install_pkg(const char *pkg, bool clean, bool global) {
 		free (needs);
 		if (error) {
 			if (r2pm_check ("apt") && r_file_is_directory ("/system/bin")) {
-				if (r_cons_yesno ('y', "Install system dependencies (Y/n)")) {
-					const char *const cmd = "apt install build-essential git make patch python wget binutils";
+				RCons *cons = r_cons_singleton ();
+				if (r_cons_yesno (cons, 'y', "Install system dependencies (Y/n)")) {
+					const char cmd[] = "apt install build-essential ninja git make patch python wget binutils";
 					R_LOG_INFO ("Running %s", cmd);
-					r_sys_cmd (cmd);
-					return r2pm_install_pkg (pkg, clean, global);
+					if (r_sys_cmd (cmd) == 0) {
+						const char cmd[] = "pip3 install meson --break-system-packages";
+						if (r_sys_cmd (cmd) == 0) {
+							return r2pm_install_pkg (pkg, clean, global);
+						}
+					}
 				}
 			}
 			return -1;
@@ -812,10 +816,10 @@ static int r2pm_install_pkg(const char *pkg, bool clean, bool global) {
 	char *qjs_script = r2pm_get (pkg, "\nR2PM_INSTALL_QJS() {\n", TT_CODEBLOCK);
 	if (qjs_script) {
 		int res = 0;
+#if R2__UNIX__ && !defined(__wasi__)
 		const char *const argv[5] = {
 			"radare2", "-j", "-e", qjs_script, NULL
 		};
-#if R2__UNIX__ && !defined(__wasi__)
 		int child = fork ();
 		if (child == -1) {
 			R_LOG_ERROR ("Cannot find radare2 in PATH");
@@ -873,8 +877,8 @@ static int r2pm_install_pkg(const char *pkg, bool clean, bool global) {
 		return 1;
 	}
 	char *s = have_builddir
-		? r_str_newf ("cd '%s'\nexport MAKE=make\nR2PM_FAIL(){\n  echo $@\n}\n%s", pkgdir, script)
-		: r_str_newf ("export MAKE=make\nR2PM_FAIL(){\n  echo $@\n}\n%s", script);
+		? r_str_newf ("cd '%s'\nexport MAKE=make\nR2PM_FAIL(){\n  echo $@\nexit 1\n}\n%s", pkgdir, script)
+		: r_str_newf ("export MAKE=make\nR2PM_FAIL(){\n  echo $@\nexit 1\n}\n%s", script);
 	// if no srcdir is defined because no file to pull just dont cd
 	free (pkgdir);
 	int res = r_sandbox_system (s, 1);
@@ -1067,7 +1071,7 @@ static int r2pm_info(void) {
 	return 0;
 }
 
-static char *r2pm_search(const char *grep) {
+static char *r2pm_search(const char *grep, int mode) {
 	char *path = r2pm_dbdir ();
 	RList *files = r_sys_dir (path);
 	free (path);
@@ -1076,20 +1080,39 @@ static char *r2pm_search(const char *grep) {
 	}
 	RListIter *iter;
 	const char *file;
-	RStrBuf *sb = r_strbuf_new ("");
+	PJ *pj = NULL;
+	RStrBuf *sb = NULL;
+	if (mode == 'j') {
+		pj = pj_new ();
+		pj_a (pj);
+	} else {
+		sb = r_strbuf_new ("");
+	}
 	r_list_foreach (files, iter, file) {
 		if (*file != '.') {
 			bool match = R_STR_ISEMPTY (grep) || r_str_casestr (file, grep);
 			char *desc = r2pm_desc (file);
 			if (desc) {
 				if (match || r_str_casestr (desc, grep)) {
-					r_strbuf_appendf (sb, "%s%s%s\n", file, r_str_pad (' ', 20 - strlen (file)), desc);
+					if (pj) {
+						pj_o (pj);
+						pj_ks (pj, "name", file);
+						pj_ks (pj, "desc", desc);
+						pj_end (pj);
+					} else {
+						r_strbuf_appendf (sb, "%s%s%s\n",
+							file, r_str_pad (' ', 20 - strlen (file)), desc);
+					}
 				}
 				free (desc);
 			}
 		}
 	}
 	r_list_free (files);
+	if (pj) {
+		pj_end (pj);
+		return pj_drain (pj);
+	}
 	return r_strbuf_drain (sb);
 }
 
@@ -1097,6 +1120,7 @@ static void r2pm_envhelp(void) {
 	int r2pm_log_level = r_sys_getenv_asint ("R2_LOG_LEVEL");
 	char *r2pm_plugdir = r_sys_getenv ("R2PM_PLUGDIR");
 	char *r2pm_bindir = r_sys_getenv ("R2PM_BINDIR");
+	char *r2pm_mandir = r_sys_getenv ("R2PM_MANDIR");
 	char *r2pm_libdir = r_sys_getenv ("R2PM_LIBDIR");
 	char *r2pm_dbdir = r_sys_getenv ("R2PM_DBDIR");
 	char *r2pm_prefix = r_sys_getenv ("R2PM_PREFIX");
@@ -1108,12 +1132,12 @@ static void r2pm_envhelp(void) {
 		"SUDO=sudo              # path to the SUDO executable\n"
 		"MAKE=make              # path to the GNU MAKE executable\n"
 		"R2PM_OFFLINE=%d         # don't git pull\n"
-		"R2PM_LEGACY=0\n"
 		"R2PM_TIME=YYYY-MM-DD\n"
 		"R2PM_PLUGDIR=%s\n"
 		"R2PM_PLUGDIR=%s (global)\n"
 		"R2PM_PREFIX=%s\n"
 		"R2PM_BINDIR=%s\n"
+		"R2PM_MANDIR=%s\n"
 		"R2PM_LIBDIR=%s\n"
 		"R2PM_DBDIR=%s\n"
 		"R2PM_GITDIR=%s\n"
@@ -1124,6 +1148,7 @@ static void r2pm_envhelp(void) {
 		r2pm_plugdir2,
 		r2pm_prefix,
 		r2pm_bindir,
+		r2pm_mandir,
 		r2pm_libdir,
 		r2pm_dbdir,
 		r2pm_gitdir,
@@ -1132,6 +1157,7 @@ static void r2pm_envhelp(void) {
 	free (r2pm_plugdir2);
 	free (r2pm_prefix);
 	free (r2pm_bindir);
+	free (r2pm_mandir);
 	free (r2pm_dbdir);
 	free (r2pm_gitdir);
 	free (r2pm_giturl);
@@ -1147,9 +1173,10 @@ static void r2pm_varprint(const char *name) {
 
 R_API int r_main_r2pm(int argc, const char **argv) {
 	bool havetoflush = false;
-	if (!r_cons_is_initialized ()) {
+	RCons *cons = r_cons_global (NULL);
+	if (!cons) {
 		havetoflush = true;
-		r_cons_new ();
+		cons = r_cons_new2 ();
 	}
 #if R2__UNIX__
 	char *wd = getcwd (NULL, 0);
@@ -1177,7 +1204,7 @@ R_API int r_main_r2pm(int argc, const char **argv) {
 		0
 	};
 	RGetopt opt;
-	r_getopt_init (&opt, argc, argv, "aqecdiIhH:flgrpst:uUv");
+	r_getopt_init (&opt, argc, argv, "aqecdiIjhH:flgrRpst:uUv");
 	int i, c;
 	bool action = false;
 	// -H option without argument
@@ -1192,6 +1219,9 @@ R_API int r_main_r2pm(int argc, const char **argv) {
 			r2pm.add = true;
 			action = true;
 			break;
+		case 'j':
+			r2pm.json = true;
+			break;
 		case 'q':
 			r2pm.quiet = true;
 			break;
@@ -1243,6 +1273,10 @@ R_API int r_main_r2pm(int argc, const char **argv) {
 		case 't':
 			r2pm.time = opt.arg;
 			break;
+		case 'R':
+			r2pm.reload = true;
+			action = true;
+			break;
 		case 'r':
 			r2pm.run = true;
 			action = true;
@@ -1292,7 +1326,9 @@ R_API int r_main_r2pm(int argc, const char **argv) {
 	}
 	if (r2pm.version) {
 		int mode = 0;
-		if (r2pm.quiet) {
+		if (r2pm.json) {
+			mode = 'j';
+		} else if (r2pm.quiet) {
 			mode = 'q';
 		}
 		return r_main_version_print ("r2pm", mode);
@@ -1347,13 +1383,17 @@ R_API int r_main_r2pm(int argc, const char **argv) {
 		res = r2pm_clean (targets);
 	}
 	if (r2pm.search) {
-		char *s = r2pm_search (argv[opt.ind]);
+		char *s = r2pm_search (argv[opt.ind], r2pm.json? 'j': 0);
 		if (s) {
-			r_cons_print (s);
-			if (havetoflush) {
-				r_cons_flush ();
+			if (*s) {
+				r_cons_print (cons, s);
+				if (havetoflush) {
+					r_cons_flush (cons);
+				}
+				res = 0;
+			} else {
+				res = 1;
 			}
-			res = 0;
 			free (s);
 		} else {
 			res = 1;
@@ -1371,17 +1411,33 @@ R_API int r_main_r2pm(int argc, const char **argv) {
 	} else if (r2pm.clean) {
 		res = r2pm_clean (targets);
 	} else if (r2pm.list) {
-		char *s = r2pm_list ();
+		char *s = r2pm_list (r2pm.json? 'j': 0);
 		if (s) {
-			r_cons_print (s);
+			r_cons_print (cons, s);
 			if (havetoflush) {
-				r_cons_flush ();
+				r_cons_flush (cons);
 			}
 			res = 0;
 		} else {
 			res = 1;
 		}
 	}
+	if (r2pm.reload) {
+		RListIter *iter;
+		const char *pkg;
+		r_list_foreach (targets, iter, pkg) {
+			char *s = r2pm_get (pkg, "\nR2PM_RELOAD() {", TT_CODEBLOCK);
+			if (s) {
+				char *t = r_str_trim_lines (s);
+				r_cons_print (cons, t);
+				free (t);
+				free (s);
+			}
+		}
+		if (havetoflush) {
+			r_cons_flush (cons);
+		}
+	}
 	r_list_free (targets);
 	if (res != -1) {
 		return res;
