@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake */
+/* radare - LGPL - Copyright 2009-2025 - pancake */
 
 #include <r_core.h>
 #include "../../libr/bin/format/pdb/pdb_downloader.h"
@@ -7,8 +7,42 @@ typedef struct rabin2_state_t {
 	char *stdin_buf;
 } Rabin2State;
 
+typedef struct {
+	const char *name;
+	const char *desc;
+} Rabin2Env;
+
+static Rabin2Env env[] = {
+	{ "R2_NOPLUGINS",				"                      # same as r2 -N. Dont load shared plugins" },
+	{ "RABIN2_ARGS",				"                      # ignore cli and use these program arguments" },
+	{ "RABIN2_CHARSET",				"e cfg.charset         # set default value charset for -z strings" },
+	{ "RABIN2_CODESIGN_VERBOSE",	"                      # show codesign details at parse time" },
+//TODO { "RABIN2_MACHO_CODESIGN",	"" },
+	{ "RABIN2_DEBASE64",			"e bin.str.debase64    # try to debase64 all strings" },
+	{ "RABIN2_DEMANGLE",			"e bin.demangle        # dont demangle symbols if value is 0" },
+	{ "RABIN2_DEMANGLE_CMD", 		"e bin.demangle.cmd    # try to purge false positives" },
+	{ "RABIN2_DEMANGLE_TRYLIB",		"e bin.demangle.trylib # load Swift libs to demangle (default: false)" },
+//	{ "RABIN2_DEMAN_PFXLIB",		"e bin.demangle.pfxlib # prefix symbols with library name" },
+//	{ "RABIN2_DEMAN_NAT",			"e bin.demangle.native # load Swift libs to demangle (default: true)" },
+	{ "RABIN2_LANG",				"e bin.lang            # assume lang for demangling" },
+	{ "RABIN2_MACHO_NOFUNCSTARTS",	"                      # if set it will ignore the FUNCSTART information" },
+	{ "RABIN2_MACHO_NOSWIFT",		"                      # avoid parsing the swift metadata" },
+	{ "RABIN2_MACHO_SKIPFIXUPS",	"                      # do not parse the mach-o chained fixups" },
+	{ "RABIN2_MAXSTRBUF",			"e bin.str.maxbuf      # specify maximum buffer size" },
+	{ "RABIN2_PDBSERVER",			"e pdb.server          # use alternative PDB server" },
+	{ "RABIN2_PREFIX",				"e bin.prefix          # prefix symbols/sections/relocs with a specific string" },
+	{ "RABIN2_STRFILTER",			"e bin.str.filter      # r2 -qc 'e bin.str.filter=?" "?' -" },
+	{ "RABIN2_STRPURGE",			"e bin.str.purge       # try to purge false positives" },
+//	{ "RABIN2_STR_FILTER",			"e bin.str.filter      # r2 -qc 'e bin.str.filter=?" "?' -" },
+//	{ "RABIN2_STR_PURGE",			"e bin.str.purge       # try to purge false positives" },"
+	{ "RABIN2_SYMSTORE",			"e pdb.symstore        # path to downstream symbol store" },
+	{ "RABIN2_VERBOSE",				"e bin.verbose         # show debugging messages from the parser" },
+};
+
+static void rabin_show_env(bool show_desc);
+
 static int rabin_show_help(int v) {
-	printf ("Usage: rabin2 [-AcdeEghHiIjlLMqrRsSUvVxzZ] [-@ at] [-a arch] [-b bits] [-B addr]\n"
+	printf ("Usage: rabin2 [-AcdeEghHiIjJlLMqrRsSUvVxzZ] [-@ at] [-a arch] [-b bits] [-B addr]\n"
 		"              [-C F:C:D] [-f str] [-m addr] [-n str] [-N m:M] [-P[-P] pdb]\n"
 		"              [-o str] [-O help] [-k query] [-D lang mangledsymbol] file\n");
 	if (v) {
@@ -29,12 +63,13 @@ static int rabin_show_help(int v) {
 		" -f [str]        select sub-bin named str\n"
 		" -F [binfmt]     force to use that bin plugin (ignore header check)\n"
 		" -g              same as -SMZIHVResizcld -SS -SSS -ee (show all info)\n"
-		" -G [addr]       load address . offset to header\n"
+		" -G [addr]       load address . address to header\n"
 		" -h              this help message\n"
 		" -H              header fields\n"
 		" -i              imports (symbols imported from libraries)\n"
 		" -I              binary info\n"
 		" -j              output in json\n"
+		" -J ([var])      display variable\n"
 		" -k [sdb-query]  run sdb query. for example: '*'\n"
 		" -K [algo]       calculate checksums (md5, sha1, ..)\n"
 		" -l              linked libraries\n"
@@ -49,7 +84,7 @@ static int rabin_show_help(int v) {
 		" -P              show debug/pdb information\n"
 		" -PP             download pdb file for binary\n"
 		" -q              be quiet, just show fewer data\n"
-		" -qq             show less info (no offset/size for -z for ex.)\n"
+		" -qq             show less info (no addr/size for -z for ex.)\n"
 		" -Q              show load address used by dlopen (non-aslr libs)\n"
 		" -r              radare output\n"
 		" -R              relocations\n"
@@ -73,32 +108,8 @@ static int rabin_show_help(int v) {
 		);
 	}
 	if (v) {
-		printf ("Environment:\n"
-		" R2_NOPLUGINS:                                  # same as r2 -N. Dont load shared plugins\n"
-		" RABIN2_ARGS:                                   # ignore cli and use these program arguments\n"
-		" RABIN2_CHARSET:          e cfg.charset         # set default value charset for -z strings\n"
-		" RABIN2_CODESIGN_VERBOSE:                       # show codesign details at parse time\n"
-		// TODO RABIN2_MACHO_CODESIGN
-		" RABIN2_DEBASE64:         e bin.str.debase64    # try to debase64 all strings\n"
-		" RABIN2_DEMANGLE:         e bin.demangle        # dont demangle symbols if value is 0\n"
-		" RABIN2_DEMANGLE_CMD:     e bin.demangle.cmd    # try to purge false positives\n"
-		" RABIN2_DEMANGLE_TRYLIB:  e bin.demangle.trylib # load Swift libs to demangle (default: false)\n"
-		// " RABIN2_DEMAN_PFXLIB:  e bin.demangle.pfxlib # prefix symbols with library name\n"
-		// " RABIN2_DEMAN_NAT: e bin.demangle.native # load Swift libs to demangle (default: true)\n"
-		" RABIN2_LANG:             e bin.lang            # assume lang for demangling\n"
-		" RABIN2_MACHO_NOFUNCSTARTS                      # if set it will ignore the FUNCSTART information\n"
-		" RABIN2_MACHO_NOSWIFT                           # avoid parsing the swift metadata\n"
-		" RABIN2_MACHO_SKIPFIXUPS                        # do not parse the mach-o chained fixups\n"
-		" RABIN2_MAXSTRBUF:        e bin.str.maxbuf      # specify maximum buffer size\n"
-		" RABIN2_PDBSERVER:        e pdb.server          # use alternative PDB server\n"
-		" RABIN2_PREFIX:           e bin.prefix          # prefix symbols/sections/relocs with a specific string\n"
-		" RABIN2_STRFILTER:        e bin.str.filter      # r2 -qc 'e bin.str.filter=?" "?' -\n"
-		" RABIN2_STRPURGE:         e bin.str.purge       # try to purge false positives\n"
-		// " RABIN2_STR_FILTER: e bin.str.filter   # r2 -qc 'e bin.str.filter=?" "?' -\n"
-		// " RABIN2_STR_PURGE:  e bin.str.purge    # try to purge false positives\n"
-		" RABIN2_SYMSTORE:         e pdb.symstore        # path to downstream symbol store\n"
-		" RABIN2_VERBOSE:          e bin.verbose         # show debugging messages from the parser\n"
-		);
+		printf ("Environment:\n");
+		rabin_show_env (true);
 	}
 	return 1;
 }
@@ -329,7 +340,7 @@ static bool __dumpSections(RBin *bin, const char *scnname, const char *output, c
 	return true;
 }
 
-static int rabin_do_operation(RBin *bin, const char *op, int rad, const char *output, const char *file) {
+static int rabin_do_operation(RCons *cons, RBin *bin, const char *op, int rad, const char *output, const char *file) {
 	char *arg = NULL, *ptr = NULL, *ptr2 = NULL;
 	bool rc = true;
 
@@ -431,8 +442,8 @@ static int rabin_do_operation(RBin *bin, const char *op, int rad, const char *ou
 		if (plg && plg->signature) {
 			char *sign = plg->signature (cur, rad == R_MODE_JSON);
 			if (sign) {
-				r_cons_println (sign);
-				r_cons_flush ();
+				r_cons_println (cons, sign);
+				r_cons_flush (cons);
 				free (sign);
 			}
 		}
@@ -470,9 +481,9 @@ error:
 	return false;
 }
 
-static int rabin_show_srcline(RBin *bin, ut64 at) {
+static bool rabin_show_srcline(RBin *bin, ut64 at) {
 	char *srcline;
-	if (at != UT64_MAX && (srcline = r_bin_addr2text (bin, at, true))) {
+	if (at != UT64_MAX && (srcline = r_bin_addrline_tostring (bin, at, 1))) {
 		printf ("%s\n", srcline);
 		free (srcline);
 		return true;
@@ -481,7 +492,7 @@ static int rabin_show_srcline(RBin *bin, ut64 at) {
 }
 
 /* bin callback */
-static int __lib_bin_cb(RLibPlugin *pl, void *user, void *data) {
+static bool __lib_bin_cb(RLibPlugin *pl, void *user, void *data) {
 	struct r_bin_plugin_t *hand = (struct r_bin_plugin_t *)data;
 	RBin *bin = user;
 	//printf(" * Added (dis)assembly plugin\n");
@@ -489,12 +500,12 @@ static int __lib_bin_cb(RLibPlugin *pl, void *user, void *data) {
 	return true;
 }
 
-static int __lib_bin_dt(RLibPlugin *pl, void *p, void *u) {
+static bool __lib_bin_dt(RLibPlugin *pl, void *p, void *u) {
 	return true;
 }
 
 /* binxtr callback */
-static int __lib_bin_xtr_cb(RLibPlugin *pl, void *user, void *data) {
+static bool __lib_bin_xtr_cb(RLibPlugin *pl, void *user, void *data) {
 	struct r_bin_xtr_plugin_t *hand = (struct r_bin_xtr_plugin_t *)data;
 	RBin *bin = user;
 	//printf(" * Added (dis)assembly plugin\n");
@@ -502,12 +513,12 @@ static int __lib_bin_xtr_cb(RLibPlugin *pl, void *user, void *data) {
 	return true;
 }
 
-static int __lib_bin_xtr_dt(RLibPlugin *pl, void *p, void *u) {
+static bool __lib_bin_xtr_dt(RLibPlugin *pl, void *p, void *u) {
 	return true;
 }
 
 /* binldr callback */
-static int __lib_bin_ldr_cb(RLibPlugin *pl, void *user, void *data) {
+static bool __lib_bin_ldr_cb(RLibPlugin *pl, void *user, void *data) {
 	struct r_bin_ldr_plugin_t *hand = (struct r_bin_ldr_plugin_t *)data;
 	RBin *bin = user;
 	//printf(" * Added (dis)assembly plugin\n");
@@ -515,18 +526,18 @@ static int __lib_bin_ldr_cb(RLibPlugin *pl, void *user, void *data) {
 	return true;
 }
 
-static int __lib_bin_ldr_dt(RLibPlugin *pl, void *p, void *u) {
+static bool __lib_bin_ldr_dt(RLibPlugin *pl, void *p, void *u) {
 	return true;
 }
 
 static char *__demangleAs(RBin *bin, int type, const char *file) {
-	bool syscmd = bin->demangle_usecmd;
+	bool syscmd = bin->options.demangle_usecmd;
 	char *res = NULL;
 	switch (type) {
 	case R_BIN_LANG_CXX: res = r_bin_demangle_cxx (NULL, file, 0); break;
 	case R_BIN_LANG_JAVA: res = r_bin_demangle_java (file); break;
 	case R_BIN_LANG_OBJC: res = r_bin_demangle_objc (NULL, file); break;
-	case R_BIN_LANG_SWIFT: res = r_bin_demangle_swift (file, syscmd, bin->demangle_trylib); break;
+	case R_BIN_LANG_SWIFT: res = r_bin_demangle_swift (file, syscmd, bin->options.demangle_trylib); break;
 	case R_BIN_LANG_MSVC: res = r_bin_demangle_msvc (file); break;
 	case R_BIN_LANG_RUST: res = r_bin_demangle_rust (NULL, file, 0); break;
 	default:
@@ -536,16 +547,34 @@ static char *__demangleAs(RBin *bin, int type, const char *file) {
 	return res;
 }
 
-static void __listPlugins(RBin *bin, const char* plugin_name, PJ *pj, int rad) {
+static void list_plugins(RBin *bin, const char* plugin_name, PJ *pj, int rad) {
 	int format = (rad == R_MODE_JSON) ? 'j': rad? 'q': 0;
 	bin->cb_printf = (PrintfCallback)printf;
-	if (plugin_name) {
+	if (R_STR_ISNOTEMPTY (plugin_name)) {
 		r_bin_list_plugin (bin, plugin_name, pj, format);
 	} else {
 		r_bin_list (bin, pj, format);
 	}
 }
 
+static void rabin_env_print(const char *name) {
+	char *value = r_sys_getenv (name);
+	printf ("%s\n", R_STR_ISNOTEMPTY (value) ? value : "");
+	free (value);
+}
+
+static void rabin_show_env(bool show_desc) {
+	int id = 0;
+	for (id = 0; id < (sizeof (env) / sizeof (env[0])); id++) {
+		if (show_desc) {
+			printf ("%s\t%s\n", env[id].name, env[id].desc);
+		} else {
+			printf ("%s=", env[id].name);
+			rabin_env_print(env[id].name);
+		}
+	}
+}
+
 R_API int r_main_rabin2(int argc, const char **argv) {
 	Rabin2State state = {0};
 	const char *name = NULL;
@@ -556,7 +585,7 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 	ut64 baddr = UT64_MAX;
 	const char *do_demangle = NULL;
 	const char *query = NULL;
-	int c, bits = 0, actions = 0;
+	int c, bits = 0;
 	char* create = NULL;
 	bool va = true;
 	ut64 action = R_BIN_REQ_UNK;
@@ -574,6 +603,7 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 
 	r_core_init (&core);
 	RBin *bin = core.bin;
+	RCons *cons = core.cons;
 
 	state.stdin_buf = malloc (STDIN_BUF_SIZE);
 	if (!state.stdin_buf) {
@@ -594,7 +624,7 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			&__lib_bin_ldr_cb, &__lib_bin_ldr_dt, bin);
 		/* load plugins everywhere */
 		char *path = r_sys_getenv (R_LIB_ENV);
-		if (path && *path) {
+		if (R_STR_ISNOTEMPTY (path)) {
 			r_lib_opendir (l, path);
 		}
 		r_lib_opendir (l, homeplugindir);
@@ -661,10 +691,16 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 	}
 
 #define is_active(x) (action & (x))
-#define set_action(x) { actions++; action |= (x); }
+#define set_action(x) { action |= (x); }
 #define unset_action(x) action &= ~x
 	RGetopt opt;
-	r_getopt_init (&opt, argc, argv, "DjgAf:F:a:B:G:b:cC:k:K:dD:Mm:n:N:@:isSVIHeEUlRwO:o:pPqQrTtvLhuxXzZ");
+	r_getopt_init (&opt, argc, argv, "DjJ:gAf:F:a:B:G:b:cC:k:K:dD:Mm:n:N:@:isSVIHeEUlRwO:o:pPqQrTtvLhuxXzZ");
+	if (argc == 2 && !strcmp (argv[1], "-J")) {
+		rabin_show_env (false);
+		r_core_fini (&core);
+		free (state.stdin_buf);
+		return 0;
+	}
 	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
 		case 'g':
@@ -678,7 +714,7 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			set_action (R_BIN_REQ_SIZE);
 			set_action (R_BIN_REQ_INFO);
 			set_action (R_BIN_REQ_FIELDS);
-			set_action (R_BIN_REQ_DWARF);
+			set_action (R_BIN_REQ_ADDRLINE);
 			set_action (R_BIN_REQ_ENTRIES);
 			set_action (R_BIN_REQ_INITFINI);
 			set_action (R_BIN_REQ_MAIN);
@@ -750,10 +786,17 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			break;
 		case 'Z': set_action (R_BIN_REQ_SIZE); break;
 		case 'I': set_action (R_BIN_REQ_INFO); break;
+		case 'J':
+			rabin_env_print (opt.arg);
+			r_core_fini (&core);
+			free (state.stdin_buf);
+			return 0;
 		case 'H':
 			set_action (R_BIN_REQ_FIELDS);
 			break;
-		case 'd': set_action (R_BIN_REQ_DWARF); break;
+		case 'd':
+			set_action (R_BIN_REQ_ADDRLINE);
+			break;
 		case 'P':
 			if (is_active (R_BIN_REQ_PDB)) {
 				set_action (R_BIN_REQ_PDB_DWNLD);
@@ -833,7 +876,15 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			}
 			break;
 		case 'B':
-			baddr = r_num_math (NULL, opt.arg);
+			{
+				const char *err = NULL;
+				//baddr = r_num_math_err (core.num, opt.arg, &err);
+				baddr = r_num_math_err (NULL, opt.arg, &err);
+				if (err) {
+					R_LOG_ERROR ("%s in base address (-B): '%s'", err, opt.arg);
+					return 1;
+				}
+			}
 			break;
 		case '@':
 			at = r_num_math (NULL, opt.arg);
@@ -853,6 +904,7 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			break;
 		case 'h':
 			r_core_fini (&core);
+			free (create);
 			free (state.stdin_buf);
 			return rabin_show_help (1);
 		default:
@@ -876,10 +928,10 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 		if (opt.ind < argc) {
 			plugin_name = argv[opt.ind];
 		}
-		__listPlugins (bin, plugin_name, pj, rad);
+		list_plugins (bin, plugin_name, pj, rad);
 		if (rad == R_MODE_JSON) {
-			r_cons_println (pj_string (pj));
-			r_cons_flush ();
+			r_cons_println (cons, pj_string (pj));
+			r_cons_flush (cons);
 			pj_free (pj);
 		}
 		r_core_fini (&core);
@@ -895,7 +947,7 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			return rabin_show_help (0);
 		}
 		int type = r_bin_demangle_type (do_demangle);
-		bin->demangle_trylib = (type == R_BIN_LANG_SWIFT && r_sys_getenv_asbool ("RABIN2_DEMAN_TRYLIB"));
+		bin->options.demangle_trylib = (type == R_BIN_LANG_SWIFT && r_sys_getenv_asbool ("RABIN2_DEMAN_TRYLIB"));
 		file = argv[opt.ind + 1];
 		if (!strcmp (file, "-")) {
 			for (;;) {
@@ -1037,7 +1089,7 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			return waitpid (child, NULL, 0);
 		}
 #endif
-		void *addr = r_lib_dl_open (file);
+		void *addr = r_lib_dl_open (file, false);
 		if (addr) {
 			R_LOG_INFO ("%s is loaded at 0x%"PFMT64x, file, (ut64)(size_t)(addr));
 			r_lib_dl_close (addr);
@@ -1100,8 +1152,8 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			return 1;
 		}
 	}
-	bin->minstrlen = r_config_get_i (core.config, "bin.str.min");
-	bin->maxstrbuf = r_config_get_i (core.config, "bin.str.maxbuf");
+	bin->options.minstrlen = r_config_get_i (core.config, "bin.str.min");
+	bin->options.maxstrbuf = r_config_get_i (core.config, "bin.str.maxbuf");
 
 	r_bin_force_plugin (bin, forcebin);
 	r_bin_load_filter (bin, action);
@@ -1130,13 +1182,13 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 		RBinFile *bf = r_bin_cur (core.bin);
 		if (bf) {
 			bf->strmode = rad;
-			r_bin_dump_strings (bf, bin->minstrlen, bf->rawstr);
+			r_bin_dump_strings (bf, bin->options.minstrlen, bf->rawstr);
 		}
 	}
 	if (query) {
 		if (rad) {
 			r_core_bin_export_info (&core, R_MODE_RADARE);
-			r_cons_flush ();
+			r_cons_flush (cons);
 		} else {
 			if (!strcmp (query, "-")) {
 				__sdb_prompt (&state, bin->cur->sdb);
@@ -1148,31 +1200,30 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 		free (state.stdin_buf);
 		return 0;
 	}
-#define isradjson ((rad == R_MODE_JSON) && (actions > 0))
-#define run_action(n,x,y) {\
+#define run_action(n, x, y) {\
 	if (action & (x)) {\
-		if (isradjson) { pj_k (pj, n); } \
+		if (pj) { pj_k (pj, n); } \
 		if (!r_core_bin_info (&core, y, pj, rad, va, &filter, chksum)) {\
 			R_LOG_ERROR ("Missing bin header %s", n);\
 		};\
 	}\
 }
 	core.bin = bin;
-	bin->cb_printf = r_cons_printf;
-	filter.offset = at;
+	bin->cb_printf = r_cons_gprintf; // XXX deprecate
+	filter.addr = at;
 	filter.name = name;
-	r_cons_new ()->context->is_interactive = false;
+	core.cons->context->is_interactive = false;
 
-	if (isradjson) {
+	if (pj) {
 		pj_o (pj);
 	}
 	// List fatmach0 sub-binaries, etc
 	if (action & R_BIN_REQ_LISTARCHS || ((arch || bits || arch_name) &&
 		!r_bin_select (bin, arch, bits, arch_name))) {
 		if (rad == R_MODE_SIMPLEST || rad == R_MODE_SIMPLE) {
-			r_bin_list_archs (bin, pj, 'q');
+			r_bin_list_archs (bin, pj, NULL, 'q');
 		} else {
-			r_bin_list_archs (bin, pj, (rad == R_MODE_JSON)? 'j': 1);
+			r_bin_list_archs (bin, pj, NULL, (rad == R_MODE_JSON)? 'j': 1);
 		}
 		free (arch_name);
 	}
@@ -1187,7 +1238,13 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 			R_FREE (tmp);
 		}
 		pdbopts.symbol_store_path = (char *)r_config_get (core.config, "pdb.symstore");
-		r_bin_pdb_download (&core, pj, isradjson, &pdbopts);
+		if (pj) {
+			pj_k (pj, "pdb");
+		}
+		r_bin_pdb_download (&core, pj, &pdbopts);
+		if (pj) {
+			pj_end (pj);
+		}
 	}
 
 	if ((tmp = r_sys_getenv ("RABIN2_PREFIX"))) {
@@ -1211,11 +1268,12 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 	run_action ("header", R_BIN_REQ_HEADER, R_CORE_BIN_ACC_HEADER);
 	run_action ("libs", R_BIN_REQ_LIBS, R_CORE_BIN_ACC_LIBS);
 	run_action ("relocs", R_BIN_REQ_RELOCS, R_CORE_BIN_ACC_RELOCS);
-	run_action ("dwarf", R_BIN_REQ_DWARF, R_CORE_BIN_ACC_DWARF);
+	run_action ("addrline", R_BIN_REQ_ADDRLINE, R_CORE_BIN_ACC_ADDRLINE);
 	run_action ("pdb", R_BIN_REQ_PDB, R_CORE_BIN_ACC_PDB);
+	// run_action ("pdb", R_BIN_REQ_PDB_DWNLD, R_CORE_BIN_ACC_PDB);
 	run_action ("size", R_BIN_REQ_SIZE, R_CORE_BIN_ACC_SIZE);
 	run_action ("versioninfo", R_BIN_REQ_VERSIONINFO, R_CORE_BIN_ACC_VERSIONINFO);
-	run_action ("sections", R_BIN_REQ_SIGNATURE, R_CORE_BIN_ACC_SIGNATURE);
+	run_action ("signature", R_BIN_REQ_SIGNATURE, R_CORE_BIN_ACC_SIGNATURE);
 	run_action ("hashes", R_BIN_REQ_HASHES, R_CORE_BIN_ACC_HASHES);
 	run_action ("sections mapping", R_BIN_REQ_SECTIONS_MAPPING, R_CORE_BIN_ACC_SECTIONS_MAPPING);
 	if (action & R_BIN_REQ_SRCLINE) {
@@ -1230,15 +1288,15 @@ R_API int r_main_rabin2(int argc, const char **argv) {
 		}
 	}
 	if (op && action & R_BIN_REQ_OPERATION) {
-		rabin_do_operation (bin, op, rad, output, file);
+		rabin_do_operation (cons, bin, op, rad, output, file);
 	}
-	if (isradjson) {
+	if (pj) {
 		pj_end (pj);
-		r_cons_println (pj_string (pj));
+		r_cons_println (cons, pj_string (pj));
 	}
 
 	pj_free (pj);
-	r_cons_flush ();
+	r_cons_flush (cons);
 	r_core_fini (&core);
 	r_syscmd_popalld ();
 	free (state.stdin_buf);
